#!/usr/bin/python
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Radek Novacek <rnovacek@redhat.com>
#

import sys
import pywbem

if len(sys.argv) < 4:
    print """Usage: %s <address> <state> [username] [password]
    Connect to CIM server at address and change the power state of the machine.

Available states:
    4 - sleep
    5 - force reboot
    7 - hibernate
    8 - force poweroff
    12 - poweroff
    15 - reboot

Example: %s https://127.0.0.1:5989 4 root redhat""" % (sys.argv[0], sys.argv[0])
    sys.exit(1)

url = sys.argv[1]
try:
    state = int(sys.argv[2])
except ValueError:
    print >>sys.stderr, "Unknown state: %s" % sys.argv[2]
    sys.exit(4)

username = None
password = None
if len(sys.argv) > 3:
    username = sys.argv[3]
if len(sys.argv) > 4:
    password = sys.argv[4]

cliconn = pywbem.WBEMConnection(url, (username, password))

computerSystems = cliconn.ExecQuery('WQL', 'select * from Linux_ComputerSystem')

if len(computerSystems) == 0:
    print >>sys.stderr, "No usable Linux_ComputerSystem instance found."
    sys.exit(2)

if len(computerSystems) > 1:
    print >>sys.stderr, "More than one Linux_ComputerSystem instance found, don't know which to use."
    sys.exit(3)

print cliconn.InvokeMethod("RequestPowerStateChange", "LMI_PowerManagementService",
                     ManagedElement=computerSystems[0].path,
                     TimeoutPeriod=pywbem.datetime.now(),
                     PowerState=pywbem.Uint16(state),
                     Time=pywbem.datetime.now()
                    )



# -*- encoding: utf-8 -*-
# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
"""
Pylint checker for CIM provider modules and classes.
"""

import re
from logilab.astng import node_classes # pylint: disable=W0403
from logilab.astng import scoped_nodes # pylint: disable=W0403
from pylint.interfaces import IASTNGChecker
from pylint.checkers import BaseChecker

_RE_PROVIDER_CLASS_NAME = re.compile(
    r'^(?P<prefix>[A-Z][a-zA-Z0-9]*)_(?P<name>[A-Z][a-zA-Z]*)$')
_RE_PROVIDER_MODULE_NAME = re.compile(
    r'^((?P<parent>.*)\.)?(?P<basename>(?P<prefix>[A-Z][a-zA-Z0-9]*)'
    r'_(?P<name>[A-Z][a-zA-Z]*))$')
_RE_COMMON_MODULE_NAME = re.compile(
    r'^([a-z_][a-z0-9_]*)|([A-Z][a-zA-Z0-9]+)$')

def recursively_clean_cls_values(linter, node):
    """
    Values class under provider defines property values, under
    nested classes that are popular target of pylint messages. Since
    these are generated and known to any developper, we don't want
    to be bothered by these warnings.

    Supress them for any nested class recursively.
    """
    # class has too few public methods
    linter.disable('R0903', scope='module', line=node.fromlineno)
    # class has not docstring
    linter.disable('C0111', scope='module', line=node.fromlineno)
    for child in node.get_children():
        if isinstance(child, scoped_nodes.Class):
            recursively_clean_cls_values(linter, child)

def supress_cim_provider_messages(linter, node):
    """
    Supress some warnings for CIMProvider2 subclass.
    @param node is a subclass of CIMProvider2
    """
    assert isinstance(node, scoped_nodes.Class)
    # class has too many public methods
    linter.disable('R0904', scope='module', line=node.fromlineno)
    if '__init__' in node:
        # __init__ not called for base class
        linter.disable('W0231', scope='module', line=node['__init__'].fromlineno)
    if (  'Values' in node
       and isinstance(node['Values'], scoped_nodes.Class)):
        recursively_clean_cls_values(linter, node['Values'])

    generated_methods = (
        'get_instance',
        'enum_instances',
        'set_instance',
        'delete_instance')
    # implementation of generated methods may not use all arguments
    for child in node.get_children():
        if (  not isinstance(child, scoped_nodes.Function)
           or (   child.name not in generated_methods
              and not child.name.startswith('cim_method_'))):
            continue
        # provider method could be a function
        linter.disable('R0201', scope='module', line=child.fromlineno)
        for arg in child.args.get_children():
            linter.disable('W0613', scope='module', line=arg.fromlineno)

class CIMProviderChecker(BaseChecker):
    """
    Checks for compliance to naming conventions for python cim providers.
    """

    __implements__ = IASTNGChecker

    name = 'cim_provider'
    msgs = {
        'C9905': ('Invalid provider module name %s',
                  "Module containing cim provider(s) should be named as "
                  "<prefix>_<name>. Where both <prefix> and <name> are "
                  "CamelCased strings."),
        'C9906': ('Prefixes of module and class does not match: %s != %s',
                  "Module prefix has to match provider class prefix."),
        'W9908': ("get_providers in module %s is not a function",
                  "get_providers should be a callable function."),
        'W9909': ("Missing provider name \"%s\" in providers dictionary.",
                  "Function get_providers returns providers association"
                  " dictionary, that must include all module's provider"
                  " classes."),
        'C9910': ("Prefix different from LMI: %s",
                  "All OpenLMI CIM classes have LMI_ ORGID prefix."),
        'C9911': ("Invalid module name: %s",
                  "All modules, that does not declare cim provider class"
                  " should be named according to this regexp:"
                  " '^(([a-z_][a-z0-9_]*)|([A-Z][a-zA-Z0-9]+))$'."),
        }

    # this is important so that your checker is executed before others
    priority = -2

    def visit_class(self, node):
        """
        Check every class, which inherits from
          pywbem.cim_provider2.CIMProvider2.
        """
        if "CIMProvider2" in [a.name for a in node.ancestors()]:
            supress_cim_provider_messages(self.linter, node)
            parent = node.parent
            while not isinstance(parent, scoped_nodes.Module):
                parent = parent.parent
            clsm = _RE_PROVIDER_CLASS_NAME.match(node.name)
            modm = _RE_PROVIDER_MODULE_NAME.match(parent.name)
            if clsm and not modm:
                self.add_message('C9905', node=node, args=parent.name)
            if clsm and modm and clsm.group('prefix') != modm.group('prefix'):
                self.add_message('C9906', node=node,
                        args=(modm.group('prefix'), clsm.group('prefix')))
            if clsm and clsm.group('prefix') != 'LMI':
                self.add_message('C9910', node=node, args=clsm.group('prefix'))
            if 'get_providers' in parent.keys():
                getprovs = parent['get_providers']
                if not isinstance(getprovs, scoped_nodes.Function):
                    self.add_message('W9908', node=getprovs, args=parent.name)
                ret = getprovs.last_child()
                if not isinstance(ret, node_classes.Return):
                    return
                dictionary = ret.get_children().next()
                if not isinstance(dictionary, node_classes.Dict):
                    return
                if not node.name in [i[0].value for i in dictionary.items]:
                    self.add_message('W9909', node=getprovs, args=node.name)

        elif node.name == "Values" and len([a for a in node.ancestors()]) == 1:
            recursively_clean_cls_values(self.linter, node)

    def visit_module(self, node):
        """
        Check for invalid module name.
        """
        modm = _RE_PROVIDER_MODULE_NAME.match(node.name)
        if modm:
            if not _RE_COMMON_MODULE_NAME.match(node.name):
                for child in node.get_children():
                    if (   isinstance(child, scoped_nodes.Class)
                       and 'CIMProvider2' in [
                           a.name for a in child.ancestors()]):
                        break
                else:
                    self.add_message('C9911', node=node, args=node.name)

def register(linter):
    """required method to auto register our checker"""
    linter.register_checker(CIMProviderChecker(linter))




# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# Authors: Michal Minar <miminar@redhat.com>
# Authors: Roman Rakus <rrakus@redhat.com>
#
"""
Base class and utilities for test suits written with plain pywbem
abastractions.
"""

import BaseHTTPServer
import pywbem
import Queue
import random
import threading
import time

from lmi.test import base

JOB_CREATED = 4096

class CIMListener(object):
    """ CIM Listener for indication events. """

    class CIMHandler(BaseHTTPServer.BaseHTTPRequestHandler):
        """ Handler for the POST request from *CIMOM*. """
        def do_POST(self):
            """ Handle the POST request. """
            data = self.rfile.read(int(self.headers['Content-Length']))
            ttree = pywbem.parse_cim(pywbem.xml_to_tupletree(data))
            # Get the instance from CIM-XML, copied from
            # http://sf.net/apps/mediawiki/pywbem/?title=Indications_Tutorial
            insts = [x[1] for x in ttree[2][2][0][2][2]]
            for inst in insts:
                self.callback(inst)
            self.send_response(200)
            self.end_headers()

        def log_message(self, _fmt, *_args):
            # suppress log messages
            pass

    def __init__(self, callback, http_port=5988):
        self.address = ('', http_port)
        self.CIMHandler.callback = callback
        self.thread = None
        self.server = None

    def start(self):
        """ Start listening. """
        BaseHTTPServer.HTTPServer.allow_reuse_address = True
        self.server = BaseHTTPServer.HTTPServer(self.address, self.CIMHandler)
        self.thread = threading.Thread(target=self.server.serve_forever)
        self.thread.start()

    def stop(self):
        """ Stop listening. """
        if self.server is not None:
            self.server.shutdown()
            self.server.socket.close()
        if self.thread is not None:
            self.thread.join()

    def running(self):
        """
        Are we listening for events?
        :rtype: bool
        """
        return self.thread is not None

class CIMTestCase(base.BaseLmiTestCase):
    """
    Base class for LMI test cases based on plain pywbem.
    """

    #: Says, whether the test case needs indication listener running or not.
    #: Each subclass shall override this property and set it to ``True`` if
    #: it want to test indication events.
    NEEDS_INDICATIONS = False

    _WBEMCONNECTION = None
    _SYSTEM_INAME = None

    @classmethod
    def needs_indications(cls):
        """
        Whether the indication listener should be started for this test case.
        In subclasses override ``NEEDS_INDICATIONS`` property and set it to
        ``True`` if indication testing is desired.
        """
        return cls.NEEDS_INDICATIONS

    @classmethod
    def setUpClass(cls):
        base.BaseLmiTestCase.setUpClass.im_func(cls)
        if cls.needs_indications():
            cls.indication_port = random.randint(12000, 13000)
            cls.indication_queue = Queue.Queue()
            cls.listener = CIMListener(
                    callback=cls._process_indication,
                    http_port=cls.indication_port)

    @classmethod
    def tearDownClass(cls):
        if cls.needs_indications():
            cls.listener.stop()

    @classmethod
    def _start_listening(cls):
        """ Start listening for incoming indications. """
        cls.listener.start()

    @classmethod
    def _process_indication(cls, indication):
        """ Callback to process one indication."""
        cls.indication_queue.put(indication)

    @property
    def wbemconnection(self):
        """
        :returns: Active connection to *CIMOM*.
        :rtype: :py:class:`pywbem.WBEMConnection`
        """
        if CIMTestCase._WBEMCONNECTION is None:
            CIMTestCase._WBEMCONNECTION = pywbem.WBEMConnection(self.url,
                    (self.username, self.password))
        return CIMTestCase._WBEMCONNECTION

    @property
    def system_iname(self):
        """
        :returns: Instance of ``CIM_ComputerSystem`` registered with *CIMOM*.
        :rtype: :py:class:`lmi.shell.LMIInstanceName`
        """
        if CIMTestCase._SYSTEM_INAME is None:
            CIMTestCase._SYSTEM_INAME = self.wbemconnection. \
                    EnumerateInstanceNames(self.system_cs_name, 'root/cimv2')[0]
        return CIMTestCase._SYSTEM_INAME.copy()

    def setUp(self):
        self.subscribed = {}

    def tearDown(self):
        for name in self.subscribed.keys():
            self.unsubscribe(name)

    def assertCIMIsSubclass(self, cls, base_cls):
        """
        Checks, whether cls is subclass of base_cls from CIM perspective.
        @param cls name of subclass
        @param base_cls name of base class
        """
        if not isinstance(cls, basestring):
            raise TypeError("cls must be a string")
        if not isinstance(base_cls, basestring):
            raise TypeError("base_cls must be a string")
        return self.assertTrue(pywbem.is_subclass(self.wbemconnection,
            "root/cimv2", base_cls, cls))

    def get_indication(self, timeout):
        """ Wait for an indication for given nr. of seconds and return it."""
        try:
            indication = self.indication_queue.get(timeout=timeout)
        except Queue.Empty:
            raise AssertionError("Timeout when waiting for indication")
        self.indication_queue.task_done()
        return indication

    def make_filter_iname(self, filter_name):
        """
        Create an instance name of ``CIM_IndicationFilter``.

        :rtype: :py:class:`pywbem.CIMInstanceName`
        """
        return pywbem.CIMInstanceName(
                    classname="CIM_IndicationFilter",
                    namespace="root/interop",
                    keybindings={
                        'CreationClassName': 'CIM_IndicationFilter',
                        'SystemClassName': self.system_cs_name,
                        'SystemName': self.SYSTEM_NAME,
                        'Name': filter_name})

    def make_filter_inst(self, filter_name, query, query_lang="DMTF:CQL"):
        """
        Create an instance of ``CIM_IndicationFilter``.

        :rtype: :py:class:`pywbem.CIMInstance`
        """
        inst = pywbem.CIMInstance('CIM_IndicationFilter')
        inst['CreationClassName'] = 'CIM_IndicationFilter'
        inst['SystemCreationClassName'] = self.system_cs_name
        inst['SystemName'] = self.SYSTEM_NAME
        inst['Name'] = filter_name
        inst['Query'] = query
        inst['QueryLanguage'] = query_lang
        inst['SourceNamespace'] = "root/cimv2"
        inst.path = self.make_filter_iname(filter_name)
        return inst

    def subscribe(self, filter_name, query=None, querylang="DMTF:CQL"):
        """
        Create indication subscription for given filter name.
        """
        if not self.needs_indications():
            raise Exception("can not subscribe to indications, enable them"
                    " with NEEDS_INDICATIONS")

        if query is not None:
            # Create filter first
            indfilter = self.wbemconnection.CreateInstance(
                    self.make_filter_inst(filter_name, query, querylang))
        else:
            # the filter is already created, assemble its name
            indfilter = self.make_filter_iname(filter_name)

        # create destination
        destinst = pywbem.CIMInstance('CIM_ListenerDestinationCIMXML')
        destinst['CreationClassName'] = 'CIM_ListenerDestinationCIMXML'
        destinst['SystemCreationClassName'] = self.system_cs_name
        destinst['SystemName'] = self.SYSTEM_NAME
        destinst['Name'] = filter_name
        destinst['Destination'] = "http://localhost:%d" % (self.indication_port)
        destinst['PersistenceType'] = pywbem.Uint16(3) # Transient
        cop = pywbem.CIMInstanceName(
                'CIM_ListenerDestinationCIMXML', namespace="root/interop")
        cop.keybindings = {
                'CreationClassName' : 'CIM_ListenerDestinationCIMXML',
                'SystemClassName'   : self.system_cs_name,
                'SystemName'        : self.SYSTEM_NAME,
                'Name'              : filter_name }
        destinst.path = cop
        destname = self.wbemconnection.CreateInstance(destinst)

        # create the subscription
        subinst = pywbem.CIMInstance(
                'CIM_IndicationSubscription')
        subinst['Filter'] = indfilter
        subinst['Handler'] = destname
        cop = pywbem.CIMInstanceName(
                'CIM_IndicationSubscription', namespace="root/interop")
        cop.keybindings = {
                'Filter': indfilter,
                'Handler': destname }
        subinst.path = cop
        subscription = self.wbemconnection.CreateInstance(subinst)

        self.subscribed[filter_name] = [subscription, destname]

        if not self.listener.running():
            self._start_listening()
        return subscription

    def unsubscribe(self, filter_name):
        """
        Unsubscribe from given filter.
        """
        if not self.needs_indications():
            raise Exception("can not unsubscribe to indications, enable them"
                    " with NEEDS_INDICATIONS")
        for instance in self.subscribed.pop(filter_name):
            self.wbemconnection.DeleteInstance(instance)

    def finish_job(self,
            jobname,
            assoc_class,
            return_constructor=int):
        """
        Wait until the job finishes and return ``(ret, outparams)``just as
        ``InvokeMethod`` would.

        It's hard to reconstruct these outparams, since the embedded
        instances / ojects do not work in our ``CIMOMS``, therefore special
        care is needed.

        :param jobname: Name of the job.
        :type jobname: :py:class:`pywbem.CIMInstanceName`
        :param callable return_constructor: Callable, which converts
            string to the right type, for example ``int``.
        :returns: ``(retvalue, outparams)`` in the same way as
            ``finish_method()`` would.
        :rtype: tuple
        """
        # Use busy loop for now
        # TODO: rework to something sane
        while True:
            job = self.wbemconnection.GetInstance(jobname)
            if job['JobState'] > 5:  # all states higher than 5 are final
                break
            time.sleep(0.1)

        # get the MethodResult
        resultname = self.wbemconnection.AssociatorNames(
                jobname,
                AssocClass=assoc_class)[0]
        result = self.wbemconnection.GetInstance(resultname)
        ind = result['PostCallIndication']
        # check for error
        if ind['Error'] is not None:
            err = ind['Error'][0]
            code = err['CIMStatusCode']
            msg = err['Message']
            raise pywbem.CIMError(code, msg)

        ret = return_constructor(ind['ReturnValue'])

        # convert output parameters to format returned by InvokeMethod
        outparams = pywbem.NocaseDict()
        try:
            params = ind['MethodParameters']
        except KeyError:
            params = {}
        if params:
            for (key, value) in params.iteritems():
                outparams[key] = value

        return (ret, outparams)

    def invoke_async_method(self,
            method_name,
            object_name,
            return_constructor=int,
            *args, **kwargs):
        """
        Invoke a method and if it returns a job, wait for the job.

        :param string method_name: Name of the method.
        :param object_name: Instance, on which the method should be invoked.
        :type object_name: :py:class:`pywbem.CIMInstanceName`
        :param callable return_constructor: Callable, which converts
            string to the right type, for example ``int``.
        :param list args: Positional arguments passed to invoked method.
        :param dictionary kwargs: Keyword arguments passed to invoked method.
        :returns: ``(retvalue, outparams)`` in the same way as
            ``finish_method()`` would.
        :rtype: tuple
        """
        (ret, outparams) = self.wbemconnection.InvokeMethod(
                method_name,
                object_name,
                *args,
                **kwargs)
        if ret == JOB_CREATED:
            # wait for the job
            jobname = outparams['Job']
            (ret, outparams) = self.finish_job(jobname, return_constructor)
        return (ret, outparams)



# -*- encoding: utf-8 -*-
# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""
Just a common functionality related to LMI_SoftwareIdentityResource provider.
"""

import pywbem

from lmi.providers import cmpi_logging
from lmi.providers import ComputerSystem
from lmi.providers import is_this_system
from lmi.software import util
from lmi.software.yumdb import YumDB
from lmi.software.yumdb.repository import Repository

LOG = cmpi_logging.get_logger(__name__)

class Values(object):
    class DetailedStatus(object):
        Not_Available = pywbem.Uint16(0)
        No_Additional_Information = pywbem.Uint16(1)
        Stressed = pywbem.Uint16(2)
        Predictive_Failure = pywbem.Uint16(3)
        Non_Recoverable_Error = pywbem.Uint16(4)
        Supporting_Entity_in_Error = pywbem.Uint16(5)
        # DMTF_Reserved = ..
        # Vendor_Reserved = 0x8000..
        _reverse_map = {
                0 : 'Not Available',
                1 : 'No Additional Information',
                2 : 'Stressed',
                3 : 'Predictive Failure',
                4 : 'Non-Recoverable Error',
                5 : 'Supporting Entity in Error'
        }

    class RequestedState(object):
        Unknown = pywbem.Uint16(0)
        Enabled = pywbem.Uint16(2)
        Disabled = pywbem.Uint16(3)
        Shut_Down = pywbem.Uint16(4)
        No_Change = pywbem.Uint16(5)
        Offline = pywbem.Uint16(6)
        Test = pywbem.Uint16(7)
        Deferred = pywbem.Uint16(8)
        Quiesce = pywbem.Uint16(9)
        Reboot = pywbem.Uint16(10)
        Reset = pywbem.Uint16(11)
        Not_Applicable = pywbem.Uint16(12)
        # DMTF_Reserved = ..
        # Vendor_Reserved = 32768..65535
        _reverse_map = {
                0  : 'Unknown',
                2  : 'Enabled',
                3  : 'Disabled',
                4  : 'Shut Down',
                5  : 'No Change',
                6  : 'Offline',
                7  : 'Test',
                8  : 'Deferred',
                9  : 'Quiesce',
                10 : 'Reboot',
                11 : 'Reset',
                12 : 'Not Applicable'
        }

    class HealthState(object):
        Unknown = pywbem.Uint16(0)
        OK = pywbem.Uint16(5)
        Degraded_Warning = pywbem.Uint16(10)
        Minor_failure = pywbem.Uint16(15)
        Major_failure = pywbem.Uint16(20)
        Critical_failure = pywbem.Uint16(25)
        Non_recoverable_error = pywbem.Uint16(30)
        # DMTF_Reserved = ..
        # Vendor_Specific = 32768..65535
        _reverse_map = {
                0  : 'Unknown',
                5  : 'OK',
                10 : 'Degraded/Warning',
                15 : 'Minor failure',
                20 : 'Major failure',
                25 : 'Critical failure',
                30 : 'Non-recoverable error'
        }

    class TransitioningToState(object):
        Unknown = pywbem.Uint16(0)
        Enabled = pywbem.Uint16(2)
        Disabled = pywbem.Uint16(3)
        Shut_Down = pywbem.Uint16(4)
        No_Change = pywbem.Uint16(5)
        Offline = pywbem.Uint16(6)
        Test = pywbem.Uint16(7)
        Defer = pywbem.Uint16(8)
        Quiesce = pywbem.Uint16(9)
        Reboot = pywbem.Uint16(10)
        Reset = pywbem.Uint16(11)
        Not_Applicable = pywbem.Uint16(12)
        # DMTF_Reserved = ..
        _reverse_map = {
                0  : 'Unknown',
                2  : 'Enabled',
                3  : 'Disabled',
                4  : 'Shut Down',
                5  : 'No Change',
                6  : 'Offline',
                7  : 'Test',
                8  : 'Defer',
                9  : 'Quiesce',
                10 : 'Reboot',
                11 : 'Reset',
                12 : 'Not Applicable'
        }

    class ResourceType(object):
        Unknown = pywbem.Uint16(0)
        Other = pywbem.Uint16(1)
        Installer_and_Payload = pywbem.Uint16(2)
        Installer = pywbem.Uint16(3)
        Payload = pywbem.Uint16(4)
        Installability_checker = pywbem.Uint16(5)
        Security_Advisory = pywbem.Uint16(6)
        Engineering_Advisory = pywbem.Uint16(7)
        Technical_release_notes = pywbem.Uint16(9)
        Change_notification = pywbem.Uint16(10)
        Whitepaper = pywbem.Uint16(11)
        Marketing_Documentation = pywbem.Uint16(12)
        # DMTF_Reserved = ..
        # Vendor_Reserved = 0x8000..0xFFFF
        _reverse_map = {
                0  : 'Unknown',
                1  : 'Other',
                2  : 'Installer and Payload',
                3  : 'Installer',
                4  : 'Payload',
                5  : 'Installability checker',
                6  : 'Security Advisory',
                7  : 'Engineering Advisory',
                9  : 'Technical release notes',
                10 : 'Change notification',
                11 : 'Whitepaper',
                12 : 'Marketing Documentation'
        }

    class EnabledState(object):
        Unknown = pywbem.Uint16(0)
        Other = pywbem.Uint16(1)
        Enabled = pywbem.Uint16(2)
        Disabled = pywbem.Uint16(3)
        Shutting_Down = pywbem.Uint16(4)
        Not_Applicable = pywbem.Uint16(5)
        Enabled_but_Offline = pywbem.Uint16(6)
        In_Test = pywbem.Uint16(7)
        Deferred = pywbem.Uint16(8)
        Quiesce = pywbem.Uint16(9)
        Starting = pywbem.Uint16(10)
        # DMTF_Reserved = 11..32767
        # Vendor_Reserved = 32768..65535
        _reverse_map = {
                0  : 'Unknown',
                1  : 'Other',
                2  : 'Enabled',
                3  : 'Disabled',
                4  : 'Shutting Down',
                5  : 'Not Applicable',
                6  : 'Enabled but Offline',
                7  : 'In Test',
                8  : 'Deferred',
                9  : 'Quiesce',
                10 : 'Starting'
        }

    class ExtendedResourceType(object):
        Unknown = pywbem.Uint16(0)
        Not_Applicable = pywbem.Uint16(2)
        Linux_RPM = pywbem.Uint16(3)
        HP_UX_Depot = pywbem.Uint16(4)
        Windows_MSI = pywbem.Uint16(5)
        Solaris_Package = pywbem.Uint16(6)
        Macintosh_Disk_Image = pywbem.Uint16(7)
        Debian_linux_Package = pywbem.Uint16(8)
        HP_Smart_Component = pywbem.Uint16(11)
        # Vendor_Reserved = 101..200
        HTML = pywbem.Uint16(201)
        PDF = pywbem.Uint16(202)
        Text_File = pywbem.Uint16(203)
        # DMTF_Reserved = ..
        # Vendor_Reserved = 0x8000..0xFFFF
        _reverse_map = {
                0   : 'Unknown',
                2   : 'Not Applicable',
                3   : 'Linux RPM',
                4   : 'HP-UX Depot',
                5   : 'Windows MSI',
                6   : 'Solaris Package',
                7   : 'Macintosh Disk Image',
                8   : 'Debian linux Package',
                201 : 'HTML',
                202 : 'PDF',
                11  : 'HP Smart Component',
                203 : 'Text File'
        }

    class AccessContext(object):
        Unknown = pywbem.Uint16(0)
        Other = pywbem.Uint16(1)
        Default_Gateway = pywbem.Uint16(2)
        DNS_Server = pywbem.Uint16(3)
        SNMP_Trap_Destination = pywbem.Uint16(4)
        MPLS_Tunnel_Destination = pywbem.Uint16(5)
        DHCP_Server = pywbem.Uint16(6)
        SMTP_Server = pywbem.Uint16(7)
        LDAP_Server = pywbem.Uint16(8)
        Network_Time_Protocol__NTP__Server = pywbem.Uint16(9)
        Management_Service = pywbem.Uint16(10)
        internet_Storage_Name_Service__iSNS_ = pywbem.Uint16(11)
        # DMTF_Reserved = ..
        # Vendor_Reserved = 32768..65535
        _reverse_map = {
                0  : 'Unknown',
                1  : 'Other',
                2  : 'Default Gateway',
                3  : 'DNS Server',
                4  : 'SNMP Trap Destination',
                5  : 'MPLS Tunnel Destination',
                6  : 'DHCP Server',
                7  : 'SMTP Server',
                8  : 'LDAP Server',
                9  : 'Network Time Protocol (NTP) Server',
                10 : 'Management Service',
                11 : 'internet Storage Name Service (iSNS)'
        }

    class AvailableRequestedStates(object):
        Enabled = pywbem.Uint16(2)
        Disabled = pywbem.Uint16(3)
        Shut_Down = pywbem.Uint16(4)
        Offline = pywbem.Uint16(6)
        Test = pywbem.Uint16(7)
        Defer = pywbem.Uint16(8)
        Quiesce = pywbem.Uint16(9)
        Reboot = pywbem.Uint16(10)
        Reset = pywbem.Uint16(11)
        # DMTF_Reserved = ..
        _reverse_map = {
                2  : 'Enabled',
                3  : 'Disabled',
                4  : 'Shut Down',
                6  : 'Offline',
                7  : 'Test',
                8  : 'Defer',
                9  : 'Quiesce',
                10 : 'Reboot',
                11 : 'Reset'
        }

    class Status(object):
        OK = 'OK'
        Error = 'Error'
        Degraded = 'Degraded'
        Unknown = 'Unknown'
        Pred_Fail = 'Pred Fail'
        Starting = 'Starting'
        Stopping = 'Stopping'
        Service = 'Service'
        Stressed = 'Stressed'
        NonRecover = 'NonRecover'
        No_Contact = 'No Contact'
        Lost_Comm = 'Lost Comm'
        Stopped = 'Stopped'

    class CommunicationStatus(object):
        Unknown = pywbem.Uint16(0)
        Not_Available = pywbem.Uint16(1)
        Communication_OK = pywbem.Uint16(2)
        Lost_Communication = pywbem.Uint16(3)
        No_Contact = pywbem.Uint16(4)
        # DMTF_Reserved = ..
        # Vendor_Reserved = 0x8000..
        _reverse_map = {
                0 : 'Unknown',
                1 : 'Not Available',
                2 : 'Communication OK',
                3 : 'Lost Communication',
                4 : 'No Contact'
        }

    class OperationalStatus(object):
        Unknown = pywbem.Uint16(0)
        Other = pywbem.Uint16(1)
        OK = pywbem.Uint16(2)
        Degraded = pywbem.Uint16(3)
        Stressed = pywbem.Uint16(4)
        Predictive_Failure = pywbem.Uint16(5)
        Error = pywbem.Uint16(6)
        Non_Recoverable_Error = pywbem.Uint16(7)
        Starting = pywbem.Uint16(8)
        Stopping = pywbem.Uint16(9)
        Stopped = pywbem.Uint16(10)
        In_Service = pywbem.Uint16(11)
        No_Contact = pywbem.Uint16(12)
        Lost_Communication = pywbem.Uint16(13)
        Aborted = pywbem.Uint16(14)
        Dormant = pywbem.Uint16(15)
        Supporting_Entity_in_Error = pywbem.Uint16(16)
        Completed = pywbem.Uint16(17)
        Power_Mode = pywbem.Uint16(18)
        Relocating = pywbem.Uint16(19)
        # DMTF_Reserved = ..
        # Vendor_Reserved = 0x8000..
        _reverse_map = {
                0  : 'Unknown',
                1  : 'Other',
                2  : 'OK',
                3  : 'Degraded',
                4  : 'Stressed',
                5  : 'Predictive Failure',
                6  : 'Error',
                7  : 'Non-Recoverable Error',
                8  : 'Starting',
                9  : 'Stopping',
                10 : 'Stopped',
                11 : 'In Service',
                12 : 'No Contact',
                13 : 'Lost Communication',
                14 : 'Aborted',
                15 : 'Dormant',
                16 : 'Supporting Entity in Error',
                17 : 'Completed',
                18 : 'Power Mode',
                19 : 'Relocating'
        }

    class OperatingStatus(object):
        Unknown = pywbem.Uint16(0)
        Not_Available = pywbem.Uint16(1)
        Servicing = pywbem.Uint16(2)
        Starting = pywbem.Uint16(3)
        Stopping = pywbem.Uint16(4)
        Stopped = pywbem.Uint16(5)
        Aborted = pywbem.Uint16(6)
        Dormant = pywbem.Uint16(7)
        Completed = pywbem.Uint16(8)
        Migrating = pywbem.Uint16(9)
        Emigrating = pywbem.Uint16(10)
        Immigrating = pywbem.Uint16(11)
        Snapshotting = pywbem.Uint16(12)
        Shutting_Down = pywbem.Uint16(13)
        In_Test = pywbem.Uint16(14)
        Transitioning = pywbem.Uint16(15)
        In_Service = pywbem.Uint16(16)
        # DMTF_Reserved = ..
        # Vendor_Reserved = 0x8000..
        _reverse_map = {
                0  : 'Unknown',
                1  : 'Not Available',
                2  : 'Servicing',
                3  : 'Starting',
                4  : 'Stopping',
                5  : 'Stopped',
                6  : 'Aborted',
                7  : 'Dormant',
                8  : 'Completed',
                9  : 'Migrating',
                10 : 'Emigrating',
                11 : 'Immigrating',
                12 : 'Snapshotting',
                13 : 'Shutting Down',
                14 : 'In Test',
                15 : 'Transitioning',
                16 : 'In Service'
        }

    class RequestStateChange(object):
        Completed_with_No_Error = pywbem.Uint32(0)
        Not_Supported = pywbem.Uint32(1)
        Unknown_or_Unspecified_Error = pywbem.Uint32(2)
        Cannot_complete_within_Timeout_Period = pywbem.Uint32(3)
        Failed = pywbem.Uint32(4)
        Invalid_Parameter = pywbem.Uint32(5)
        In_Use = pywbem.Uint32(6)
        # DMTF_Reserved = ..
        Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
        Invalid_State_Transition = pywbem.Uint32(4097)
        Use_of_Timeout_Parameter_Not_Supported = pywbem.Uint32(4098)
        Busy = pywbem.Uint32(4099)
        # Method_Reserved = 4100..32767
        # Vendor_Specific = 32768..65535
        class RequestedState(object):
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Shut_Down = pywbem.Uint16(4)
            Offline = pywbem.Uint16(6)
            Test = pywbem.Uint16(7)
            Defer = pywbem.Uint16(8)
            Quiesce = pywbem.Uint16(9)
            Reboot = pywbem.Uint16(10)
            Reset = pywbem.Uint16(11)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 32768..65535

    class EnabledDefault(object):
        Enabled = pywbem.Uint16(2)
        Disabled = pywbem.Uint16(3)
        Not_Applicable = pywbem.Uint16(5)
        Enabled_but_Offline = pywbem.Uint16(6)
        No_Default = pywbem.Uint16(7)
        Quiesce = pywbem.Uint16(9)
        # DMTF_Reserved = ..
        # Vendor_Reserved = 32768..65535
        _reverse_map = {
                2 : 'Enabled',
                3 : 'Disabled',
                5 : 'Not Applicable',
                6 : 'Enabled but Offline',
                7 : 'No Default',
                9 : 'Quiesce'
        }

    class PrimaryStatus(object):
        Unknown = pywbem.Uint16(0)
        OK = pywbem.Uint16(1)
        Degraded = pywbem.Uint16(2)
        Error = pywbem.Uint16(3)
        # DMTF_Reserved = ..
        # Vendor_Reserved = 0x8000..
        _reverse_map = {
                0 : 'Unknown',
                1 : 'OK',
                2 : 'Degraded',
                3 : 'Error'
        }

    class InfoFormat(object):
        Other = pywbem.Uint16(1)
        Host_Name = pywbem.Uint16(2)
        IPv4_Address = pywbem.Uint16(3)
        IPv6_Address = pywbem.Uint16(4)
        IPX_Address = pywbem.Uint16(5)
        DECnet_Address = pywbem.Uint16(6)
        SNA_Address = pywbem.Uint16(7)
        Autonomous_System_Number = pywbem.Uint16(8)
        MPLS_Label = pywbem.Uint16(9)
        IPv4_Subnet_Address = pywbem.Uint16(10)
        IPv6_Subnet_Address = pywbem.Uint16(11)
        IPv4_Address_Range = pywbem.Uint16(12)
        IPv6_Address_Range = pywbem.Uint16(13)
        Dial_String = pywbem.Uint16(100)
        Ethernet_Address = pywbem.Uint16(101)
        Token_Ring_Address = pywbem.Uint16(102)
        ATM_Address = pywbem.Uint16(103)
        Frame_Relay_Address = pywbem.Uint16(104)
        URL = pywbem.Uint16(200)
        FQDN = pywbem.Uint16(201)
        User_FQDN = pywbem.Uint16(202)
        DER_ASN1_DN = pywbem.Uint16(203)
        DER_ASN1_GN = pywbem.Uint16(204)
        Key_ID = pywbem.Uint16(205)
        Parameterized_URL = pywbem.Uint16(206)
        # DMTF_Reserved = ..
        # Vendor_Reserved = 32768..65535
        _reverse_map = {
                1   : 'Other',
                2   : 'Host Name',
                3   : 'IPv4 Address',
                4   : 'IPv6 Address',
                5   : 'IPX Address',
                6   : 'DECnet Address',
                7   : 'SNA Address',
                8   : 'Autonomous System Number',
                9   : 'MPLS Label',
                10  : 'IPv4 Subnet Address',
                11  : 'IPv6 Subnet Address',
                12  : 'IPv4 Address Range',
                13  : 'IPv6 Address Range',
                200 : 'URL',
                201 : 'FQDN',
                202 : 'User FQDN',
                203 : 'DER ASN1 DN',
                204 : 'DER ASN1 GN',
                205 : 'Key ID',
                206 : 'Parameterized URL',
                100 : 'Dial String',
                101 : 'Ethernet Address',
                102 : 'Token Ring Address',
                103 : 'ATM Address',
                104 : 'Frame Relay Address'
        }

@cmpi_logging.trace_function
def object_path2repo(env, op, kind='enabled'):
    """
    @param op must contain precise information of repository,
    otherwise an error is raised
    """
    if not isinstance(kind, basestring):
        raise TypeError("kind must be a string")
    if not isinstance(op, pywbem.CIMInstanceName):
        raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                "op must be an instance of CIMInstanceName")

    if (  (not "CreationClassName" in op or not op['CreationClassName'])
       or (not "Name" in op or not op["Name"])
       or (  not "SystemCreationClassName" in op
          or not op["SystemCreationClassName"])
       or (not "SystemName" in op or not op["SystemName"])):
        raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER, "Wrong keys.")
    if not is_this_system(op["SystemName"]):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                'SystemName "%s" does not match "%s".' % (
                    op["SystemName"], ComputerSystem.get_system_name(env)))
    ch = env.get_cimom_handle()
    if not ch.is_subclass(util.Configuration.get_instance().namespace,
            sub=op["CreationClassName"],
            super="CIM_SoftwareIdentityResource"):
        raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                'CreationClassName \"%s\" must be a subclass of "%s".' % (
                    op["CreationClassName"], "CIM_SoftwareIdentityResource"))
    if not ch.is_subclass(util.Configuration.get_instance().namespace,
            sub=op["SystemCreationClassName"],
            super="CIM_ComputerSystem"):
        raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                'SystemCreationClassName of \"%s\" must be a subclass of "%s".'
                % (op["CreationClassName"], "CIM_SoftwareIdentityResource"))
    repos = YumDB.get_instance().filter_repositories(kind, repoid=op["Name"])
    if len(repos) < 1:
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                'No matching repository found for Name=\"%s\".' % op["Name"])
    return repos[0]

@cmpi_logging.trace_function
def _fill_non_keys(repo, model):
    """
    Fills into the model of instance all non-key properties.
    """
    model['AccessContext'] = Values.AccessContext.Other
    access_info = None
    if repo.mirror_list:
        access_info = repo.mirror_list
    elif repo.metalink:
        access_info = repo.metalink
    elif repo.base_urls:
        if len(repo.base_urls) > 0:
            if len(repo.base_urls) > 1:
                LOG().warn('multiple base urls found for repository "%s",'
                        ' selecting the last one', repo)
            access_info = repo.base_urls[-1]
    if access_info is None:
        LOG().error('no base url found for repository "%s"', repo)
        access_info = pywbem.CIMProperty('AccessInfo', None, type='string')
    model["AccessInfo"] = access_info
    model['AvailableRequestedStates'] = [
            Values.AvailableRequestedStates.Enabled,
            Values.AvailableRequestedStates.Disabled]
    model['Caption'] = repo.name
    model['Cost'] = pywbem.Sint32(repo.cost)
    model['Description'] = "[%s] - %s for %s architecture with cost %d" % (
            repo.repoid, repo.name, repo.basearch, repo.cost)
    model['ElementName'] = repo.repoid
    model['EnabledDefault'] = Values.EnabledDefault.Not_Applicable
    if repo.enabled:
        model['EnabledState'] = Values.EnabledState.Enabled
    else:
        model['EnabledState'] = Values.EnabledState.Disabled
    model['ExtendedResourceType'] = Values.ExtendedResourceType.Linux_RPM
    model['GPGCheck'] = repo.gpg_check
    if repo.ready:
        model['HealthState'] = Values.HealthState.OK
    else:
        model['HealthState'] = Values.HealthState.Major_failure
    if repo.revision is not None:
        model["Generation"] = pywbem.Uint64(repo.revision)
    else:
        model['Generation'] = pywbem.CIMProperty('Generation',
                None, type='uint64')
    model['InfoFormat'] = Values.InfoFormat.URL
    model['InstanceID'] = 'LMI:LMI_SoftwareIdentityResource:' + repo.repoid
    if repo.mirror_list:
        model["MirrorList"] = repo.mirror_list
    else:
        model['MirrorList'] = pywbem.CIMProperty('MirrorList',
                None, type='string')
    model['OperationalStatus'] = [ Values.OperationalStatus.OK
                if repo.ready else Values.OperationalStatus.Error]
    model['OtherAccessContext'] = "YUM package repository"
    model['OtherResourceType'] = "RPM Software Package"
    # this would need to populateSack, which is expensive
    #model["PackageCount"] = pywbem.Uint32(repo.pkg_count)
    if repo.ready:
        model['PrimaryStatus'] = Values.PrimaryStatus.OK
    else:
        model['PrimaryStatus'] = Values.PrimaryStatus.Error
    model['RepoGPGCheck'] = repo.repo_gpg_check
    if repo.enabled:
        model['RequestedState'] = Values.RequestedState.Enabled
    else:
        model['RequestedState'] = Values.RequestedState.Disabled
    model['ResourceType'] = Values.ResourceType.Other
    model['StatusDescriptions'] = [ "Ready" if repo.ready else "Not Ready" ]
    model['TimeOfLastStateChange'] = pywbem.CIMDateTime(repo.last_edit)
    if repo.last_update is not None:
        model['TimeOfLastUpdate'] = pywbem.CIMDateTime(repo.last_update)
    else:
        model['TimeOfLastUpdate'] = pywbem.CIMProperty('TimeOfLastUpdate',
                None, type='datetime')
    model['TransitioningToState'] = Values.TransitioningToState.Not_Applicable

@cmpi_logging.trace_function
def repo2model(env, repo, keys_only=True, model=None):
    """
    @param model if not None, will be filled with data, otherwise
    a new instance of CIMInstance or CIMObjectPath is created
    """
    if not isinstance(repo, Repository):
        raise TypeError("pkg must be an instance of Repository")
    if model is None:
        model = util.new_instance_name("LMI_SoftwareIdentityResource")
        if not keys_only:
            model = pywbem.CIMInstance(
                    "LMI_SoftwareIdentityResource", path=model)
    if isinstance(model, pywbem.CIMInstance):
        def _set_key(k, value):
            """Sets the value of key property of cim instance"""
            model[k] = value
            model.path[k] = value #pylint: disable=E1103
    else:
        _set_key = model.__setitem__
    _set_key('CreationClassName', "LMI_SoftwareIdentityResource")
    _set_key("Name", repo.repoid)
    _set_key("SystemCreationClassName",
            util.Configuration.get_instance().system_class_name)
    _set_key("SystemName", ComputerSystem.get_system_name(env))
    if not keys_only:
        _fill_non_keys(repo, model)

    return model




# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#

"""
Core functionality of particular providers.
Each provider having functionality useful to others has a submodule
in this subpackage with the same name except for LMI_ prefix.
"""

import pywbem

from lmi.providers import cmpi_logging

@cmpi_logging.trace_function
def generate_references(
        env,
        object_name,
        model,
        filter_result_class_name,
        filter_role,
        filter_result_role,
        keys_only,
        handlers):
    """
    Function for generating references to object_name. It's supposed
    to be used directly from references method of provider classes.
    @param handlers is a list of tuples:
        [ (role, class_name, handler), ... ]
      where handler is a function, that will be called for matching
        parameters, yielding instance(s/names). It accepts arguments:
            env, object_name, model, keys_only.
    """
    if not isinstance(object_name, pywbem.CIMInstanceName):
        raise TypeError("object_name must be a CIMInstanceName")
    if not isinstance(model, pywbem.CIMInstance):
        raise TypeError("model must be a CIMInstance")

    ch = env.get_cimom_handle()
    model.path.update(dict((t[0], None) for t in handlers))

    try:
        for i, (role, clsname, handler) in enumerate(handlers):
            if (  (   filter_role
                  and filter_role.lower() != role.lower())
               or not ch.is_subclass(object_name.namespace,
                            sub=object_name.classname,
                            super=clsname)):
                continue
            other_results = list((htuple[0], htuple[1]) for htuple in (
                handlers[:i] + handlers[i+1:]))
            for res_role, res_clsname in other_results:
                if (  (  not filter_result_role
                      or filter_result_role.lower() == res_role.lower())
                   and (not filter_result_class_name or ch.is_subclass(
                                object_name.namespace,
                                sub=filter_result_class_name,
                                super=res_clsname))):
                    break
            else:
                continue
            # matches filters
            for obj in handler(env, object_name, model, keys_only):
                yield obj
    except pywbem.CIMError as exc:
        if exc.args[0] != pywbem.CIM_ERR_NOT_FOUND:
            raise




# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Roman Rakus <rrakus@redhat.com>
#

from common import AccountBase
import time
from methods import *

class TestIndications(AccountBase):
    """
    Class for testing LMI_Account indications
    """
    NEEDS_INDICATIONS = True

    def test_check_good_filter(self):
        """
        Account: Test good indication filter
        """
        filter_name = "test_good_filter_%d" % (time.time() * 1000000)
        sub = self.subscribe(filter_name, "select * from LMI_AccountInstanceCreationIndication where SourceInstance isa LMI_Account")
        self.assertIsNotNone(sub)
        self.unsubscribe(filter_name);

    def test_check_bad_filter(self):
        """
        Account: Test bad indication filter
        """
        pass

    def test_group_deletion_indication(self):
        """
        Account: Test indication of group deletion
        """
        create_group(self.group_name)
        filter_name = "test_delete_group_%d" % (time.time() * 1000000)
        sub = self.subscribe(filter_name, "select * from LMI_AccountInstanceDeletionIndication where SourceInstance isa LMI_Group")
        clean_group(self.group_name)
        indication = self.get_indication(10)
        self.assertEqual(indication.classname, "LMI_AccountInstanceDeletionIndication")
        self.assertIn("SourceInstance", indication.keys())
        self.assertTrue(indication["SourceInstance"] is not None)
        self.assertEqual(indication["SourceInstance"]["Name"], self.group_name)

    def test_group_creation_indication(self):
        """
        Account: Test indication of group creation
        """
        clean_group(self.group_name)
        filter_name = "test_create_group_%d" % (time.time() * 1000000)
        sub = self.subscribe(filter_name, "select * from LMI_AccountInstanceCreationIndication where SourceInstance isa LMI_Group")
        create_group(self.group_name)
        indication = self.get_indication(10)
        self.assertEqual(indication.classname, "LMI_AccountInstanceCreationIndication")
        self.assertIn("SourceInstance", indication.keys())
        self.assertTrue(indication["SourceInstance"] is not None)
        self.assertEqual(indication["SourceInstance"]["Name"], self.group_name)
        clean_group(self.group_name)

    def test_account_deletion_indication(self):
        """
        Account: Test indication of account deletion
        """
        create_account(self.user_name)
        filter_name = "test_delete_account_%d" % (time.time() * 1000000)
        sub = self.subscribe(filter_name, "select * from LMI_AccountInstanceDeletionIndication where SourceInstance isa LMI_Account")
        clean_account(self.user_name)
        indication = self.get_indication(10)
        self.assertEqual(indication.classname, "LMI_AccountInstanceDeletionIndication")
        self.assertIn("SourceInstance", indication.keys())
        self.assertTrue(indication["SourceInstance"] is not None)
        self.assertEqual(indication["SourceInstance"]["Name"], self.user_name)

    def test_account_creation_indication(self):
        """
        Account: Test indication of account creation
        """
        clean_account(self.user_name)
        filter_name = "test_create_account_%d" % (time.time() * 1000000)
        sub = self.subscribe(filter_name, "select * from LMI_AccountInstanceCreationIndication where SourceInstance isa LMI_Account")
        create_account(self.user_name)
        indication = self.get_indication(10)
        self.assertEqual(indication.classname, "LMI_AccountInstanceCreationIndication")
        self.assertIn("SourceInstance", indication.keys())
        self.assertTrue(indication["SourceInstance"] is not None)
        self.assertEqual(indication["SourceInstance"]["Name"], self.user_name)
        clean_account(self.user_name)




# -*- encoding: utf-8 -*-
# Copyright(C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or(at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Robin Hack <rhack@redhat.com>; Jan Grec <jgrec@redhat.com>
#

import unittest
import subprocess

import lmi.shell as lmishell
import os
import shutil
import time
import pywbem # for CIMError

HOST =      os.environ.get("LMI_CIMOM_URL", "localhost")
USER =      os.environ.get("LMI_CIMOM_USERNAME", "root")
PASSWD =    os.environ.get("LMI_CIMOM_PASSWORD", "blank")
BROKER =    os.environ.get("LMI_CIMOM_BROKER", "tog-pegasus")

DEV_NULL = open("/dev/null", 'w')

# NOTES
# - systemd/service status returns code 3 for stopped (dead) services on Fedora

class TestServiceProvider(unittest.TestCase):
    """
    TestCase class testing OpenLMI service functionality.
    """

    def setUp(self):
        """Set up and check connection to remote server."""
        self.conn = lmishell.connect(HOST, USER, PASSWD)
        self.assertTrue(isinstance(self.conn, lmishell.LMIConnection), "Couldn't connect to remote provider")


    def tearDown(self):
        pass


    def test_list_services(self):
        """
        Check all services have LMI_Service instance with correct status.

        Compare all LMI_Services with output of:
            systemctl list-unit-files -t service
            systemctl status
        Cases:
            LMI_Service.instances() returns proper list of services
            service.Started returns correct value for each service against systemctl
        """
        # Get list of services from OpenLMI service provider
        lmi_services_insts = self.conn.root.cimv2.LMI_Service.instances()
        self.assertTrue(lmi_services_insts, "No services returned")

        lmi_services = dict((s.Name, s.Started) for (s) in lmi_services_insts)

        # Get list of services from systemd
        sysd_output = subprocess.check_output("systemctl list-unit-files -t service --no-pager --no-legend".split())

        #  Parse list of services. Filter blank lines and services with @.service string in name.
        sysd_services = []
        for line in sysd_output.split("\n"):
            service = line.split()
            if service and "@.service" not in service[0]:
                sysd_services.append(service)

        sysd_services = dict((s[0], False) for (s) in sysd_services)

        # Compare both lists (systemd list and OpenLMI list)
        self.assertEqual(set(sysd_services.keys()), set(lmi_services.keys()))

        # For each service check its OpenLMI status against systemd
        for service_name in sysd_services.keys():

            cmd = "systemctl status " + service_name
            ret = subprocess.call(cmd.split());

            sysd_state = (ret == 0)
            lmi_state = False
            try:
                    lmi_state = lmi_services[service_name]
            except KeyError:
                    self.fail("Service " + service_name + " is not in list of services returned by OpenLMI")

            self.assertEqual(sysd_state, lmi_state, "Different states of " + service_name)


    def test_service_up_down(self):
        """
        Test that LMI_Service instance properly stops and starts service.

        Use 'cups' service for testing.
        Compare results with:
            service cups status
        Cases:
            stopped service can be started
            running service can be stopped
        """
        # Make sure the service is not running
        subprocess.call("service cups stop".split())
        service_status_ret = subprocess.call("service cups status".split())
        self.assertEqual(service_status_ret, 3)

        service = self.conn.root.cimv2.LMI_Service.first_instance({"Name": "cups.service"})

        self.assertFalse(service.Started, "Service is not in stopped state")

        # Try to start up the service and check it's running
        service_status_stored = service.Started
        service.StartService()
        service.refresh()
        self.assertTrue(service.Started, "Service is not started")

        service_status_ret = subprocess.call("service cups status".split(), stdout=DEV_NULL, stderr=DEV_NULL)
        self.assertEqual(service_status_ret, 0)

        self.assertNotEqual(service_status_stored, service.Started)

        # Assuming the service is running from pervious code
        # Try to stop the service and check it's stopped
        service_status_stored = service.Started
        service.StopService()
        service.refresh()
        service_status_ret = subprocess.call("service cups status".split())

        self.assertFalse(service.Started)
        self.assertEqual(service_status_ret, 3)
        self.assertNotEqual(service_status_stored, service.Started)


    def test_stopped_service_restart(self):
        """
        Test that LMI_Service instance starts stopped service on restart.

        Use 'cups' service for testing.
        Compare results with:
            pidof cupsd - pid has changed
            service cups status - systemctl confirms OpenLMI status
        Cases:
            stopped service is running after restart under another pid
        """
        # Make sure the service is not running
        service_status = subprocess.call("service cups stop".split())
        self.assertEqual(service_status, 0, "Service is not stopped or is in unknown state")

        service_pid_before = 0

        # Try to start up the service by using RestartService()
        service = self.conn.root.cimv2.LMI_Service.first_instance({"Name": "cups.service"})
        service.RestartService()
        service.refresh()
        self.assertTrue(service.Started, "Service couldn't be started BACK after RestartService() [OpenLMI]")

        # Check that pid of the service has changed - service was really
        # physically stopped and started again
        service_pid = subprocess.check_output("pidof cupsd".split())
        self.assertNotEqual(service_pid_before, service_pid, "Pids of cupsd service are same as before restart.")

        service_status = subprocess.call("service cups status".split(), stdout=DEV_NULL, stderr=DEV_NULL)
        self.assertEqual(service_status, 0, "Service couldn't be started BACK after RestartService() [systemctl]")


    def test_started_service_restart(self):
        """
        Test that LMI_Service instance properly restarts running service.

        Use 'cups' service for testing.
        Compare results with:
            pidof cupsd - pid has changed
            service cups status - systemctl confirms OpenLMI status
        Cases:
            running service is running again under different pid after restart
        """
        # Make sure the service is running
        service_status = subprocess.call("service cups start".split())
        self.assertEqual(service_status, 0, "Service is not started or is in unknown state")
        service_pid_before = subprocess.check_output("pidof cupsd".split())

        # Try to restart the service by using RestartService()
        service = self.conn.root.cimv2.LMI_Service.first_instance({"Name": "cups.service"})
        service.RestartService()

        self.assertTrue(service.Started, "Service couldn't be started UP after RestartService() [OpenLMI]")

        # Check that pid of the service has changed - service was really
        # physically stopped and started again
        service_pid = subprocess.check_output("pidof cupsd".split())
        self.assertNotEqual(service_pid_before, service_pid, "Pids of cupsd service are same as before restart.")

        service_status = subprocess.call("service cups status".split(), stdout=DEV_NULL, stderr=DEV_NULL)
        self.assertEqual(service_status, 0, "Service couldn't be started UP after RestartService() [service]")


    def test_service_nonexist(self):
        """
        Test that LMI_Service can't initialize unexisting service.

        Use '..!non_exists">"' service for testing
        Compare results with:
            None - the only value that can be returned
        Cases:
            unexisting service can't be initialized (returned from LMI_Service)
        """

        # Make sure that ..!non_exists">" service really doesn't exist
        service_evil_name = "..!non_exists\">\""
        cmd = "service " + service_evil_name + " status"
        cmd = cmd.split()
        service_status = subprocess.call(cmd)
        self.assertEqual(service_status, 3)

        # Try to get ..!non_exists">" service from OpenLMI
        service = self.conn.root.cimv2.LMI_Service.first_instance({"Name": service_evil_name + ".service"})
        self.assertEqual(service, None, "Non None object returned of non-existing service")

    def test_restart_service_broker(self):
        """
        Test if is possible to restart broker over OpenLMI
        """
        service = self.conn.root.cimv2.LMI_Service.first_instance({"Name": BROKER + ".service"})
        service.RestartService()

        # Wait up to 10 seconds for the CIMOM to start
        timer = 10
        while timer > 0:
            time.sleep(1)
            try:
                service = self.conn.root.cimv2.LMI_Service.first_instance({"Name": BROKER + ".service"})
                if service is not None:
                    break
            except pywbem.CIMError:
                pass
            timer -= 1
        self.assertGreater(timer, 0, "Timeout while waiting for the cimom to restart.")

        self.assertTrue (service.Started)

        # now try to cumunicate with broker
        servicecups = self.conn.root.cimv2.LMI_Service.first_instance({"Name": "cups.service"})
        self.assertNotEqual(servicecups, None)
        servicecups.RestartService()
        servicecups.refresh()
        self.assertTrue (servicecups.Started)

    @unittest.skip("This tests breaks Pegasus.")
    def test_service_with_null_name(self):
        """
        Test that LMI_Service can't initialize "empty" service.

        Use '\0' (null character) service for testing
        Compare results with:
            None - the only value that can be returned
        Cases:
            the "null" service can't be initialized (returned from LMI_Service)
        """
        service = self.conn.root.cimv2.LMI_Service.first_instance({"Name": "\0"})
        self.assertEqual(service, None, "Non None object returned of non-existing service")


    def test_service_stopped_try_restart(self):
        """
        Test LMI_Service.TryRestartService() on stopped service.

        Use 'cups' service for testing.
        Compare results with:
            service cups status - systemctl confirms OpenLMI status
        Cases:
            stopped service is not running after TryRestartService
        """
        # Make sure the service is not running
        service_status = subprocess.call("service cups stop".split())
        self.assertEqual(service_status, 0, "Service is not stopped or is in unknown state")

        service = self.conn.root.cimv2.LMI_Service.first_instance({"Name": "cups.service"})
        self.assertFalse(service_status, "Service is not in stopped state")

        # Check that service can't be started by using TryRestartService()
        service.TryRestartService()
        self.assertFalse(service.Started, "Stopped service has been started by try-restart")
        service_status_ret = subprocess.call("service cups status".split())
        self.assertEqual(service_status_ret, 3, "Service is in running state")


    def test_service_started_try_restart(self):
        """
        Test LMI_Service.TryRestartService() on running service.

        Use 'cups' service for testing.
        Compare results with:
            pidof cupsd - pid has changed
            service cups status - systemctl confirms OpenLMI status
        Cases:
            running service is running under different pid after TryRestartService
        """
        # Make sure the service is running
        service_status = subprocess.call("service cups start".split())
        self.assertEqual(service_status, 0, "Service is not started or is in unknown state")
        service_pid_before = subprocess.check_output("pidof cupsd".split())

        # Try the TryRestartService and check that pid has changed - service
        # war physically restarted
        service = self.conn.root.cimv2.LMI_Service.first_instance({"Name": "cups.service"})
        service.TryRestartService()
        self.assertTrue(service.Started, "Try-restarted service NOT running")

        service_pid = subprocess.check_output("pidof cupsd".split())
        self.assertNotEqual(service_pid_before, service_pid, "Pids of cupsd service are same as before restart.")

        service_status = subprocess.call("service cups status".split(), stdout=DEV_NULL, stderr=DEV_NULL)
        self.assertEqual(service_status, 0, "Started service not running after try-restart")


    def test_service_get_status_when_stoppped(self):
        """
        Test that LMI_Service.started gets status when service is not running.

        Use 'cups' service for testing.
        Compare results with:
            LMI_Service.Started value
        Cases:
            stopped service returns proper status in OpenLMI
            running service returns proper status in OpenLMI
        """

        # Make sure the service is not running and try to get status
        service_status = subprocess.call("service cups stop".split())
        self.assertEqual(service_status, 0, "Service is not stopped or is in unknown state")

        service = self.conn.root.cimv2.LMI_Service.first_instance({"Name": "cups.service"})
        self.assertFalse(service.Started, "Service is not in stopped state")

        # Make sure the service is running and try to get status
        service_status = subprocess.call("service cups start".split())
        self.assertEqual(service_status, 0, "Service is not started or is in unknown state")

        service.refresh()
        self.assertTrue(service.Started, "Service is not in started state")


    # This TestCase returns errors. Ticket link:
    # https://fedorahosted.org/openlmi/ticket/81
    @unittest.skip("This will never work")
    def test_service_more_instances_status_when_stoppped(self):
        """
        Test LMI_Service gives the same results for multiple instances of one service.

        Use 'cups' service for testing.
        Compare results with:
            service.Stared only and between themselves
        Cases:
            service status is the same for multiple instances of one service
        """
        # Make sure the service is not running
        service_status = subprocess.call("service cups stop".split())
        self.assertEqual(service_status, 0, "Service is not stopped or is in unknown state")

        # Get two instances of the same service and check both statuses are False - not running
        service = self.conn.root.cimv2.LMI_Service.first_instance({"Name": "cups.service"})
        self.assertFalse(service.Started, "Service is not in stopped state")

        service2 = self.conn.root.cimv2.LMI_Service.first_instance({"Name": "cups.service"})
        self.assertFalse(service2.Started, "Service is not in stopped state")

        # Start up one service instance and check both statuses are True - running
        service.StartService()
        self.assertTrue(service.Started, "Service not started")

        self.assertEqual (service.Started, service2.Started, "States of instances are not equal")


    def test_service_turn_on_off(self):
        """
        Test LMI_Service.TurnServiceOff() and LMI_Service.TurnServiceOn()

        Use 'cups' service for testing.
        Compare results with:
            systemctl is-enabled cups.service - service was enabled/disabled
            systemctl status cups.service - service remains in previous state
        Cases:
            service is still running after disabled - TurnServiceOff()
            service is still stopped after enabled - TurnServiceOn()
        """
        # Start the service, disable it and check if it's disabled
        # and service is still running
        service = self.conn.root.cimv2.LMI_Service.first_instance({"Name": "cups.service"})
        service.StartService()
        service.refresh()
        service.TurnServiceOff()
        disabled = subprocess.call(["systemctl", "is-enabled", "cups.service"],
                                   stdout=DEV_NULL, stderr=DEV_NULL)
        self.assertEqual(disabled, 1,
                         "Service couldn't be disabled [systemctl]")
        self.assertTrue(service.Started,
                        "Service is not running after disabled [OpenLMI]")
        running = subprocess.call(["systemctl", "status", "cups.service"],
                                  stdout=DEV_NULL, stderr=DEV_NULL)
        self.assertEqual(running, 0,
                         "Service is not running after disabled [systemctl]")

        # Stop service, enable it and check if it's enabled
        # and service is still not running
        service.StopService()
        service.TurnServiceOn()
        service.refresh()
        disabled = subprocess.call(["systemctl", "is-enabled", "cups.service"],
                                   stdout=DEV_NULL, stderr=DEV_NULL)
        self.assertEqual(disabled, 0,
                         "Service couldn't be enabled [systemctl]")
        self.assertFalse(service.Started,
                         "Service is not stopped after enabled [OpenLMI]")
        stopped = subprocess.call(["systemctl", "status", "cups.service"],
                                  stdout=DEV_NULL, stderr=DEV_NULL)
        self.assertEqual(stopped, 3,
                         "Service is not stopped after enabled [systemctl]")


class TestServiceProviderFailingService(unittest.TestCase):
    """
    TestCase class testing LMI_Service functionality against constantly failing service.
    """

    def setUp(self):
        """
        Set up connection to remote server and load failing.service.

        Failing service is copied to systemd and systemctl is reloaded.
        """
        self.conn = lmishell.connect(HOST, USER, PASSWD)
        self.assertTrue(isinstance(self.conn, lmishell.LMIConnection), "Couldn't connect to remote provider")
        shutil.copy2(os.path.dirname(os.path.abspath(__file__)) + "/failing.service", "/etc/systemd/system/failing.service")
        subprocess.call ("systemctl daemon-reload".split())

    def tearDown(self):
        try:
            os.unlink ("/etc/systemd/system/failing.service")
        except OSError:
            # failing.service may be removed by test_service_race_condition
            pass
        subprocess.call ("systemctl daemon-reload".split())

    def test_failing_service_start_stop(self):
        """
        Test StartService() and StopService() on failing service.

        Use 'failing' service for testing
        Compare results with:
            service failing status - systemctl confirms OpenLMI status
        Cases:
            failing service is not running after StartService
            failing service is not running after StopService
            failing service is not running after TryRestartService
        """
        # Try to start failing service and check OpenLMI knows it failed
        service = self.conn.root.cimv2.LMI_Service.first_instance({"Name": "failing.service"})
        service.StartService()
        self.assertFalse (service.Started)
        service_status = subprocess.call("service failing status".split())
        self.assertEqual(service_status, 3, "Failing service is in different state")

        # Try to stop failing service and check OpenLMI knows it failed
        service.StopService()
        self.assertFalse (service.Started)
        service_status = subprocess.call("service failing status".split())
        self.assertEqual(service_status, 3, "Failing service is in different state")

        # TryRestartService() and check OpenLMI knows it failed
        service.TryRestartService()
        self.assertFalse (service.Started)
        service_status = subprocess.call("service failing status".split())
        self.assertEqual(service_status, 3, "Failing service is in different state")


    # Following test case can be dangerous to run on your system
    def test_service_race_condition(self):
        """
        Test various methods on systemd-unlinked failing service.

        Use 'failing' service for testing
        Compare results with:
            service.Started
        Cases:
            failing service is not running after StartService
            failing service is not running after StopService
            failing service is not running after TryRestartService
            failing service is not running after TurnServiceOff
            failing service is not running after TurnServiceOn
        """
        # Get the failing service instance from LMI_Service, unlink
        # the service from systemd and reload the daemon
        service = self.conn.root.cimv2.LMI_Service.first_instance({"Name": "failing.service"})
        os.unlink ("/etc/systemd/system/failing.service")
        subprocess.call ("systemctl daemon-reload".split())

        # All the following shoud let service.Started in False state
        # Although the instance was initialized, the original service was
        # unlinked from systemd
        service.StartService()
        self.assertFalse(service.Started)

        service.StopService()
        self.assertFalse(service.Started)

        service.TryRestartService()
        self.assertFalse(service.Started)

        service.TurnServiceOff()
        self.assertFalse(service.Started)

        service.TurnServiceOn()
        self.assertFalse(service.Started)


if __name__ == '__main__':
    # Run the standard unittest.main() function to load and run all tests
    unittest.main()



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details. #
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Radek Novacek <rnovacek@redhat.com>
# Authors: Michal Minar <miminar@redhat.com>
"""
Runs all defined tests.

Test modules are expected to be in directory with this script and
should contain subclasses of unittest.TestCase, that will be loaded.
Preferably a ``suite()`` function should be defined there as well.
They must be named according to shell regexp: ``"test_*.py"``.
"""

import argparse
import copy
import getpass
import inspect
import pywbem
import os
import sys
import tempfile
import unittest

import reposetup
import swbase

class NotFound(Exception):
    """Raised when test of particular name could no be found."""
    def __init__(self, name):
        Exception.__init__(self, "Test \"%s\" could not be found!" % name)

def parse_cmd_line():
    """
    Use ArgumentParser to parse command line arguments.

    :returns: ``(parsed arguments object, arguments for unittest.main())``.
    :rtype: tuple
    """
    parser = argparse.ArgumentParser(
            add_help=False,     # handle help message ourselves
            description="Test OpenLMI Software providers. Arguments"
            " for unittest main function can be added after \"--\""
            " switch.")
    parser.add_argument("--url",
            default=os.environ.get("LMI_CIMOM_URL", "http://localhost:5988"),
            help="Network port to use for tests")
    parser.add_argument("-u", "--user",
            default=os.environ.get("LMI_CIMOM_USERNAME", ''),
            help="User for broker authentication.")
    parser.add_argument("-p", "--password",
            default=os.environ.get("LMI_CIMOM_PASSWORD", ''),
            help="User's password.")

    dangerous_group = parser.add_mutually_exclusive_group()
    dangerous_group.add_argument("--run-dangerous", action="store_true",
            default=(os.environ.get('LMI_RUN_DANGEROUS', '0') == '1'),
            help="Run also tests dangerous for this machine"
                 " (tests, that manipulate with software database)."
                 " Overrides environment variable LMI_RUN_DANGEROUS.")
    dangerous_group.add_argument('--no-dangerous', action="store_false",
            dest="run_dangerous",
            default=os.environ.get('LMI_RUN_DANGEROUS', '0') == '1',
            help="Disable dangerous tests.")

    tedious_group = parser.add_mutually_exclusive_group()
    tedious_group.add_argument("--run-tedious", action="store_true",
            default=(os.environ.get('LMI_RUN_TEDIOUS', '0') == '1'),
            help="Run also tedious (long running) for this machine."
                 " Overrides environment variable LMI_RUN_TEDIOUS.")
    tedious_group.add_argument('--no-tedious', action="store_false",
            dest="run_tedious",
            default=os.environ.get('LMI_RUN_TEDIOUS', '0') == '1',
            help="Disable tedious tests.")

    cleanup_group = parser.add_mutually_exclusive_group()
    cleanup_group.add_argument("--cleanup-cache", action="store_true",
            default=os.environ.get('LMI_SOFTWARE_CLEANUP_CACHE', '1') == '1',
            help="Clean up all temporary files created for testing purposes."
            " If LMI_SOFTWARE_DB_CACHE is set and this is set to False, next"
            " run will be much faster because testing database won't need to"
            " be generated again. Overrides environment variable"
            " LMI_SOFTWARE_CLEANUP_CACHE.")
    cleanup_group.add_argument('--no-cleanup', action="store_false",
            dest="cleanup_cache",
            default=os.environ.get('LMI_SOFTWARE_CLEANUP_CACHE', '1') == '1',
            help="Do not delete database cache file and created repositories."
            " This speeds up testing because testing repositories don't need"
            " to be recreated upon next run. This applies only when"
            " LMI_SOFTWARE_DB_CACHE is set.")

    parser.add_argument('-l', '--list-tests', action="store_true",
            help="List all possible test names.")
    parser.add_argument('-h', '--help', action="store_true",
            help="Show help message.")

    argv = copy.copy(sys.argv)
    rest = []
    if '--' in argv:
        index = argv.index('--')
        argv = argv[:index]
        rest = sys.argv[index + 1:]
    args, unknown_args = parser.parse_known_args(argv)
    if args.help:
        parser.print_help()
        print
        print "*"*79
        print " Unit test options (parsed after \"--\" switch)."
        print "*"*79
        print
        unittest.main(argv=[argv[0], "--help"])
    return (args, unknown_args + rest)

def try_connection(args):
    """
    Try to connect to cim broker. If authentication fails, ask
    the user for credentials in loop.

    :returns: ``(user, password)``
    :rtype: tuple
    """
    user = args.user
    password = args.password
    while False:
        try:
            pywbem.WBEMConnection(args.url, (user, password))
            break
        except pywbem.cim_http.AuthError as exc:
            user = args.user
            sys.stderr.write("Authentication error\n")
            if exc.args[0] == pywbem.CIM_ERR_ACCESS_DENIED:
                if not user:
                    user = raw_input("User: ")
                password = getpass.getpass()

def prepare_environment(args):
    """
    Set the environment for test scripts.

    :returns: Whether the database cache needs to be deleted.
    :rtype: boolean
    """
    os.environ['LMI_CIMOM_URL'] = args.url
    os.environ['LMI_CIMOM_USERNAME'] = args.user
    os.environ['LMI_CIMOM_PASSWORD'] = args.password
    os.environ['LMI_RUN_DANGEROUS'] = (
            '1' if args.run_dangerous else '0')
    os.environ["LMI_RUN_TEDIOUS"] = (
            '1' if args.run_tedious else '0')
    db_cache = os.environ.get("LMI_SOFTWARE_DB_CACHE", None)
    needs_cleanup = args.cleanup_cache or not bool(db_cache)
    if not db_cache or not os.path.exists(db_cache):
        repos_dir = tempfile.mkdtemp()
        repodb, other_repos = reposetup.make_object_database(repos_dir)
        db_cache = reposetup.save_test_database(
                repos_dir, repodb, other_repos, db_cache)
    os.environ["LMI_SOFTWARE_DB_CACHE"] = db_cache
    os.environ['LMI_SOFTWARE_CLEANUP_CACHE'] = (
            '1' if needs_cleanup else '0')
    return needs_cleanup

def load_tests(loader, standard_tests, pattern):
    """
    Helper function for ``unittest.main()`` test loader.

    :returns: :py:class:`unittest.TestSuite` instance.
    """
    this_dir = os.path.dirname(__file__)
    if standard_tests:
        suite = standard_tests
    else:
        suite = unittest.TestSuite()
    discover_kwargs = dict(start_dir=this_dir)
    if pattern is not None:
        discover_kwargs['pattern'] = pattern
    tests = loader.discover(**discover_kwargs)
    suite.addTests(tests)
    return suite

class LMITestLoader(unittest.TestLoader):
    """
    Customized test loader to make invoking particular tests a lot easier.
    """

    @staticmethod
    def find_in_test_nodes(node, name):
        """
        Traverses suite tree nodes to find a test named name.

        :param string name: Name of test to find.
        :returns: Desired :py:class:`TestCase` or test function.
        """
        if (   isinstance(node, unittest.TestSuite)
           and node.countTestCases() > 0
           and isinstance(next(iter(node)), unittest.TestCase)
           and next(iter(node)).__class__.__name__ == name):
            return node
        for subnode in node:
            if isinstance(subnode, unittest.TestSuite):
                subnode = LMITestLoader.find_in_test_nodes(subnode, name)
                if subnode is not None:
                    return subnode
            elif isinstance(subnode, unittest.TestCase):
                if (  name == subnode.__class__.__name__
                   or name == subnode._testMethodName):
                    return subnode
            elif inspect.isfunction(subnode) and name == subnode.__name__:
                return subnode

    def loadTestsFromName(self, name, module=None):     #pylint: disable=C0103
        """
        Override of parent method to make test cases accessible to by their
        names from command line.
        """
        suite = load_tests(self, None, 'test_*')
        parts = name.split('.')
        node = suite
        for part in parts:
            node = LMITestLoader.find_in_test_nodes(node, part)
            if node is None:
                raise NotFound(name)
        return node

def unwind_test_suite_tree(node):
    """
    Make a list of test names out of TestSuite.

    :param node: A test suite.
    :type node: :py:class:`unittest.TestSuite`
    :returns: ``[ test_name, ... ]``
    """
    result = []
    for subnode in node:
        if isinstance(subnode, unittest.TestSuite):
            result.extend(unwind_test_suite_tree(subnode))
        elif isinstance(subnode, unittest.TestCase):
            result.append( subnode.__class__.__name__
                         + '.'+subnode._testMethodName)
        elif inspect.isfunction(subnode):
            result.append(subnode.__name__)
    return result

def list_tests():
    """
    Prints all available tests to stdout and exits.
    """
    suite = load_tests(LMITestLoader(), None, None)
    test_names = unwind_test_suite_tree(suite)
    print "\n".join(test_names)
    sys.exit(0)

def main():
    """
    Main functionality of script.
    """
    args, ut_args = parse_cmd_line()
    if args.list_tests:
        list_tests()
    try_connection(args)
    cleanup = prepare_environment(args)
    swbase.SwTestCase.setUpClass()
    try:
        test_program = unittest.main(__name__, argv=ut_args,
                testLoader=LMITestLoader(), exit=False)
    finally:
        swbase.SwTestCase.tearDownClass()
    sys.exit(0 if test_program.result.wasSuccessful() else 1)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#
"""
Unit tests for ``LMI_HostedSoftwareIdentityResource`` provider.
"""

import unittest

import swbase

ENABLED_STATE_DISABLED = 3
ENABLED_STATE_ENABLED = 2

class TestHostedSoftwareIdentityResource(swbase.SwTestCase):
    """
    Basic cim operations test on ``LMI_HostedSoftwareIdentityResource.``
    """

    CLASS_NAME = "LMI_HostedSoftwareIdentityResource"
    KEYS = ("Antecedent", "Dependent")

    def make_op(self, repo):
        """
        :returns Object path of ``LMI_HostedSoftwareIdentityResource``.
        :rtype: :py:class:`lmi.shell.LMIInstanceName`
        """
        return self.cim_class.new_instance_name({
            "Antecedent" : self.system_iname,
            "Dependent" : self.ns.LMI_SoftwareIdentityResource. \
                new_instance_name({
                    "CreationClassName"       : "LMI_SoftwareIdentityResource",
                    "SystemCreationClassName" : self.system_cs_name,
                    "SystemName"              : self.SYSTEM_NAME,
                    "Name"                    : repo.repoid
                })
            })

    @swbase.test_with_repos(**{
        'stable'          : True,
        'updates'         : True,
        'updates-testing' : False,
        'misc'            : False
    })
    def test_get_instance(self):
        """
        Test ``GetInstance()`` call on ``LMI_HostedSoftwareIdentityResource``.
        """
        for repo in self.repodb.values():
            objpath = self.make_op(repo)
            inst = objpath.to_instance()
            self.assertNotEqual(inst, None,
                    "GetInstance succeeds for repo %s" % repo.repoid)
            self.assertCIMNameEqual(objpath, inst.path,
                    "Object paths should match for repo %s" % repo.repoid)
            self.assertEqual(set(inst.path.key_properties()), set(self.KEYS))
            for key in self.KEYS:
                self.assertCIMNameEqual(
                        getattr(inst, key), getattr(inst.path, key),
                        'Key property "%s" does not match for repo "%s"!' %
                        (key, repo.repoid))

    def test_enum_instance_names(self):
        """
        Test ``EnumInstanceNames()`` call on
        ``LMI_HostedSoftwareIdentityResource``.
        """
        inames = self.cim_class.instance_names()
        repos = set(r for r in self.repodb)
        repos.update([r for r in self.other_repos])
        self.assertEqual(len(inames), len(repos))
        for iname in inames:
            self.assertIn(iname.Dependent.Name, repos)
            repoid = iname.Dependent.Name
            if repoid in self.repodb:
                repo = self.repodb[repoid]
            else:
                repo = self.other_repos[repoid]
            objpath = self.make_op(repo)
            self.assertCIMNameEqual(iname, objpath)
            self.assertEqual(set(iname.key_properties()), set(self.KEYS))
            repos.remove(iname.Dependent.Name)
        self.assertEqual(len(repos), 0)

    def test_enum_instances(self):
        """
        Test ``EnumInstanceNames()`` call on
        ``LMI_HostedSoftwareIdentityResource``.
        """
        insts = self.cim_class.instances()
        repos = set(r for r in self.repodb)
        repos.update([r for r in self.other_repos])
        self.assertEqual(len(insts), len(repos))
        for inst in insts:
            repoid = inst.Dependent.Name
            self.assertIn(repoid, repos)
            if repoid in self.repodb:
                repo = self.repodb[repoid]
            else:
                repo = self.other_repos[repoid]
            objpath = self.make_op(repo)
            self.assertCIMNameEqual(inst.path, objpath)
            self.assertEqual(set(inst.path.key_properties()), set(self.KEYS))
            for key in self.KEYS:
                self.assertCIMNameEqual(
                        getattr(inst, key), getattr(inst.path, key),
                        'Key property "%s" does not match for repo "%s"!' %
                        (key, repoid))
            repos.remove(repoid)
        self.assertEqual(len(repos), 0)

    @swbase.test_with_repos(**{
        'stable' : True,
        'updates' : True,
        'updates-testing' : False,
        'misc' : False
    })
    def test_get_computer_system_repositories(self):
        """
        Try to get repositories associated with computer system.
        """
        objpath = self.make_op(self.get_repo('stable'))
        refs = objpath.Antecedent.to_instance().associators(
                AssocClass=self.CLASS_NAME,
                Role="Antecedent",
                ResultRole="Dependent",
                ResultClass="LMI_SoftwareIdentityResource")
        repos = set(r for r in self.repodb)
        repos.update(self.other_repos.keys())
        self.assertEqual(len(refs), len(repos))
        for ref in refs:
            self.assertEqual(ref.path.namespace, 'root/cimv2')
            self.assertEqual(ref.classname, "LMI_SoftwareIdentityResource")
            repoid = ref.Name
            if repoid in self.repodb:
                repo = self.repodb[repoid]
            else:
                repo = self.other_repos[repoid]
            objpath = self.make_op(repo)
            self.assertCIMNameEqual(ref.path, objpath.Dependent)
            self.assertEqual(set(ref.path.key_properties()),
                    set(["CreationClassName", "Name",
                          "SystemCreationClassName", "SystemName"]))
            if repo.status:
                self.assertEqual(ref.EnabledState, ENABLED_STATE_ENABLED,
                        "EnabledState does not match for repo %s" % repo.repoid)
            else:
                self.assertEqual(ref.EnabledState, ENABLED_STATE_DISABLED,
                        "EnabledState does not match for repo %s" % repo.repoid)
            repos.remove(ref.Name)
        self.assertEqual(len(repos), 0)

    @swbase.test_with_repos(**{
        'stable' : True,
        'updates' : True,
        'updates-testing' : False,
        'misc' : False
    })
    def test_get_computer_system_repository_names(self):
        """
        Try to get repository names associated with computer system.
        """
        objpath = self.make_op(self.get_repo('stable'))
        refs = objpath.Antecedent.to_instance().associator_names(
                AssocClass=self.CLASS_NAME,
                Role="Antecedent",
                ResultRole="Dependent",
                ResultClass="LMI_SoftwareIdentityResource")
        repos = set(r for r in self.repodb)
        repos.update(self.other_repos.keys())
        self.assertEqual(len(refs), len(repos))
        for ref in refs:
            repoid = ref.Name
            if repoid in self.repodb:
                repo = self.repodb[repoid]
            else:
                repo = self.other_repos[repoid]
            objpath = self.make_op(repo)
            self.assertCIMNameEqual(objpath.Dependent, ref)
            self.assertEqual(set(ref.key_properties()),
                    set(["CreationClassName", "Name",
                          "SystemCreationClassName", "SystemName"]))
            repos.remove(ref.Name)
        self.assertEqual(len(repos), 0)

    @swbase.test_with_repos(**{
        'stable' : True,
        'updates' : True,
        'updates-testing' : False,
        'misc' : False
    })
    def test_get_repository_computer_systems(self):
        """
        Try to get computer system associated with repository.
        """
        for repo in self.repodb.values():
            objpath = self.make_op(repo=repo)
            refs = objpath.Dependent.to_instance().associators(
                    AssocClass=self.CLASS_NAME,
                    Role="Dependent",
                    ResultRole="Antecedent",
                    ResultClass=self.system_cs_name)
            self.assertEqual(len(refs), 1)
            self.assertCIMNameEqual(refs[0].path, objpath.Antecedent)

    @swbase.test_with_repos(**{
        'stable' : True,
        'updates' : True,
        'updates-testing' : False,
        'misc' : False
    })
    def test_get_repository_computer_system_names(self):
        """
        Try to get computer system names associated with repository.
        """
        for repo in self.repodb.values():
            objpath = self.make_op(repo=repo)
            refs = objpath.Dependent.to_instance().associator_names(
                    AssocClass=self.CLASS_NAME,
                    Role="Dependent",
                    ResultRole="Antecedent",
                    ResultClass=self.system_cs_name)
            self.assertEqual(len(refs), 1)
            self.assertCIMNameEqual(refs[0], objpath.Antecedent)

def suite():
    """For unittest loaders."""
    return unittest.TestLoader().loadTestsFromTestCase(
            TestHostedSoftwareIdentityResource)

if __name__ == '__main__':
    unittest.main()



# -*- encoding: utf-8 -*-
# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""
Just a common functionality related to
LMI_SoftwareInstallationServiceCapabilities provider.
"""

import pywbem

from lmi.providers import cmpi_logging
from lmi.software import util

class Values(object):
    class SupportedExtendedResourceTypes(object):
        Unknown = pywbem.Uint16(0)
        Other = pywbem.Uint16(1)
        Not_Applicable = pywbem.Uint16(2)
        Linux_RPM = pywbem.Uint16(3)
        HP_UX_Depot = pywbem.Uint16(4)
        Windows_MSI = pywbem.Uint16(5)
        Solaris_Package = pywbem.Uint16(6)
        Macintosh_Disk_Image = pywbem.Uint16(7)
        Debian_linux_Package = pywbem.Uint16(8)
        VMware_vSphere_Installation_Bundle = pywbem.Uint16(9)
        VMware_Software_Bulletin = pywbem.Uint16(10)
        HP_Smart_Component = pywbem.Uint16(11)
        # DMTF_Reserved = ..
        # Vendor_Reserved = 0x8000..
        _reverse_map = {
                0  : 'Unknown',
                1  : 'Other',
                2  : 'Not Applicable',
                3  : 'Linux RPM',
                4  : 'HP-UX Depot',
                5  : 'Windows MSI',
                6  : 'Solaris Package',
                7  : 'Macintosh Disk Image',
                8  : 'Debian linux Package',
                9  : 'VMware vSphere Installation Bundle',
                10 : 'VMware Software Bulletin',
                11 : 'HP Smart Component'
        }

    class SupportedInstallOptions(object):
        Defer_target_system_reset = pywbem.Uint16(2)
        Force_installation = pywbem.Uint16(3)
        Install = pywbem.Uint16(4)
        Update = pywbem.Uint16(5)
        Repair = pywbem.Uint16(6)
        Reboot = pywbem.Uint16(7)
        Password = pywbem.Uint16(8)
        Uninstall = pywbem.Uint16(9)
        Log = pywbem.Uint16(10)
        SilentMode = pywbem.Uint16(11)
        AdministrativeMode = pywbem.Uint16(12)
        ScheduleInstallAt = pywbem.Uint16(13)
        # DMTF_Reserved = ..
        # Vendor_Specific = 32768..65535
        _reverse_map = {
                2  : 'Defer target/system reset',
                3  : 'Force installation',
                4  : 'Install',
                5  : 'Update',
                6  : 'Repair',
                7  : 'Reboot',
                8  : 'Password',
                9  : 'Uninstall',
                10 : 'Log',
                11 : 'SilentMode',
                12 : 'AdministrativeMode',
                13 : 'ScheduleInstallAt'
        }

    class SupportedURISchemes(object):
        data = pywbem.Uint16(2)
        file = pywbem.Uint16(3)
        ftp = pywbem.Uint16(4)
        http = pywbem.Uint16(5)
        https = pywbem.Uint16(6)
        nfs = pywbem.Uint16(7)
        tftp = pywbem.Uint16(8)
        # DMTF_Reserved = ..
        # Vendor_Specific = 0x8000..0xFFFF
        _reverse_map = {
                2 : 'data',
                3 : 'file',
                4 : 'ftp',
                5 : 'http',
                6 : 'https',
                7 : 'nfs',
                8 : 'tftp'
        }

    class SupportedAsynchronousActions(object):
        None_supported = pywbem.Uint16(2)
        Install_From_Software_Identity = pywbem.Uint16(3)
        Install_from_ByteStream = pywbem.Uint16(4)
        Install_from_URI = pywbem.Uint16(5)
        _reverse_map = {
                2 : 'None supported',
                3 : 'Install From Software Identity',
                4 : 'Install from ByteStream',
                5 : 'Install from URI'
        }

    class CreateGoalSettings(object):
        Success = pywbem.Uint16(0)
        Not_Supported = pywbem.Uint16(1)
        Unknown = pywbem.Uint16(2)
        Timeout = pywbem.Uint16(3)
        Failed = pywbem.Uint16(4)
        Invalid_Parameter = pywbem.Uint16(5)
        Alternative_Proposed = pywbem.Uint16(6)
        # DMTF_Reserved = ..
        # Vendor_Specific = 32768..65535

    class SupportedSynchronousActions(object):
        None_supported = pywbem.Uint16(2)
        Install_From_Software_Identity = pywbem.Uint16(3)
        Install_from_ByteStream = pywbem.Uint16(4)
        Install_from_URI = pywbem.Uint16(5)
        _reverse_map = {
                2 : 'None supported',
                3 : 'Install From Software Identity',
                4 : 'Install from ByteStream',
                5 : 'Install from URI'
        }

@cmpi_logging.trace_function
def get_path():
    """@return instance name with prefilled properties"""
    op = util.new_instance_name("LMI_SoftwareInstallationServiceCapabilities",
            InstanceID="LMI:LMI_SoftwareInstallationServiceCapabilities")
    return op

@cmpi_logging.trace_function
def check_path(env, caps, prop_name):
    """
    Checks instance name of SoftwareInstallationServiceCapabilities.
    @param prop_name name of object name; used for error descriptions
    """
    if not isinstance(caps, pywbem.CIMInstanceName):
        raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                "\"%s\" must be a CIMInstanceName" % prop_name)
    our_caps = get_path()
    ch = env.get_cimom_handle()
    if caps.namespace != our_caps.namespace:
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                'Namespace of "%s" does not match "%s"' % (
                    prop_name, our_caps.namespace))
    if not ch.is_subclass(our_caps.namespace,
            sub=caps.classname,
            super=our_caps.classname):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                "Class of \"%s\" must be a sublass of %s" % (
                    prop_name, our_caps.classname))
    if caps['InstanceID'] != our_caps['InstanceID']:
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                "InstanceID of \"%s\" does not match \"%s\"" %
                prop_name, our_caps['InstanceID'])
    return True

@cmpi_logging.trace_function
def check_path_property(env, op, prop_name):
    """
    Checks, whether prop_name property of op object path is correct.
    If not, an exception will be raised.
    """
    if not prop_name in op:
        raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                "Missing %s key property!" % prop_name)
    return check_path(env, op[prop_name], prop_name)

@cmpi_logging.trace_function
def get_instance(model=None):
    """
    Makes instance of LMI_SoftwareInstallationServiceCapabilities or
    fills given model with properties.
    """
    path = get_path()
    if model is None:
        model = pywbem.CIMInstance(
                "LMI_SoftwareInstallationServiceCapabilities", path=path)
    model['InstanceID'] = path['InstanceID']

    model['CanAddToCollection'] = True
    model['Caption'] = 'Capabilities of LMI:LMI_SoftwareInstallationService'
    model['Description'] = ('This instance provides information'
        ' about LMI:LMI_SoftwareInstallationService\'s capabilities.')
    model['SupportedAsynchronousActions'] = [
        Values.SupportedAsynchronousActions.Install_From_Software_Identity,
        Values.SupportedAsynchronousActions.Install_from_URI]
    model['SupportedExtendedResourceTypes'] = [
        Values.SupportedExtendedResourceTypes.Linux_RPM]
    model['SupportedInstallOptions'] = [
        Values.SupportedInstallOptions.Install,
        Values.SupportedInstallOptions.Update,
        Values.SupportedInstallOptions.Uninstall,
        Values.SupportedInstallOptions.Force_installation,
        Values.SupportedInstallOptions.Repair]
    model['SupportedSynchronousActions'] = [
        Values.SupportedSynchronousActions.None_supported]
    model['SupportedTargetTypes'] = ['rpm', 'yum']
    model['SupportedURISchemes'] = [
            Values.SupportedURISchemes.file,
            Values.SupportedURISchemes.ftp,
            Values.SupportedURISchemes.http,
            Values.SupportedURISchemes.https]
    return model




#!/usr/bin/env python
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#
"""
Unit tests for ``LMI_MemberOfSoftwareCollection`` provider.
"""

import pywbem
import unittest

import swbase

class TestHostedSoftwareCollection(swbase.SwTestCase):
    """
    Basic cim operations test on ``LMI_HostedSoftwareCollection``.
    """

    CLASS_NAME = "LMI_HostedSoftwareCollection"
    KEYS = ("Antecedent", "Dependent")

    def make_op(self):
        """
        :returns Object path of ``LMI_HostedSoftwareCollection``.
        :rtype: :py:class:`lmi.shell.LMIInstanceName`
        """
        return self.cim_class.new_instance_name({
            "Antecedent" : self.system_iname,
            "Dependent" : self.ns.LMI_SystemSoftwareCollection. \
                new_instance_name({
                    "InstanceID" : "LMI:LMI_SystemSoftwareCollection"
                })
            })

    def test_get_instance(self):
        """
        Test ``GetInstance()`` call on ``LMI_HostedSoftwareCollection``.
        """
        objpath = self.make_op()
        inst = objpath.to_instance()
        self.assertNotEqual(inst, None)
        self.assertCIMNameEqual(inst.path, objpath)
        self.assertEqual(sorted(inst.properties()), sorted(self.KEYS))
        self.assertCIMNameEqual(objpath, inst.path)
        for key in self.KEYS:
            self.assertCIMNameEqual(getattr(inst, key), getattr(inst.path, key))

        # try with CIM_ prefix
        objpath.Antecedent.wrapped_object.classname = "CIM_ComputerSystem"

        inst = objpath.to_instance()
        self.assertNotEqual(inst, None)

        # try with CIM_ prefix also for CreationClassName
        objpath.Antecedent.wrapped_object["CreationClassName"] = \
                "CIM_ComputerSystem"

        inst = objpath.to_instance()
        self.assertNotEqual(inst, None)

    def test_enum_instances(self):
        """
        Test ``EnumInstances()`` call on ``LMI_HostedSoftwareCollection``.
        """
        objpath = self.make_op()
        insts = self.cim_class.instances()
        self.assertEqual(1, len(insts))
        self.assertCIMNameEqual(objpath, insts[0].path)
        self.assertEqual(sorted(insts[0].properties()), sorted(self.KEYS))
        for key in self.KEYS:
            self.assertCIMNameEqual(
                    getattr(insts[0], key), getattr(insts[0].path, key))

    def test_enum_instance_names(self):
        """
        Test ``EnumInstanceNames`` call on ``LMI_HostedSoftwareCollection``.
        """
        objpath = self.make_op()
        inames = self.cim_class.instance_names()
        self.assertEqual(1, len(inames))
        self.assertCIMNameEqual(objpath, inames[0])

    def test_get_computer_system_collections(self):
        """
        Try to get collection associated to computer system.
        """
        objpath = self.make_op()
        refs = objpath.Antecedent.to_instance().associator_names(
                AssocClass=self.CLASS_NAME,
                Role="Antecedent",
                ResultRole="Dependent",
                ResultClass='LMI_SystemSoftwareCollection')
        self.assertEqual(len(refs), 1)
        ref = refs[0]
        self.assertCIMNameEqual(objpath.Dependent, ref)

    def test_get_collection_computer_systems(self):
        """
        Try to get computer system associated to software collection.
        """
        objpath = self.make_op()
        refs = objpath.Dependent.to_instance().associator_names(
                AssocClass=self.CLASS_NAME,
                Role="Dependent",
                ResultRole="Antecedent",
                ResultClass=self.system_cs_name)
        self.assertEqual(1, len(refs))
        ref = refs[0]
        self.assertCIMNameEqual(objpath.Antecedent, ref)

def suite():
    """For unittest loaders."""
    return unittest.TestLoader().loadTestsFromTestCase(
            TestHostedSoftwareCollection)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Radek Novacek <rnovacek@redhat.com>
#

import os
import sys
import pywbem

HISTORY = os.path.expanduser("~/.cql-history")

class QueryException(Exception):
    pass

class Printer(object):
    def printTable(self, f, headers, table):
        raise NotImplementedError

class RawPrinter(Printer):
    def __init__(self, hsep="-", vsep="|", cross_sep="+"):
        self.hsep = hsep
        self.vsep = vsep
        self.cross_sep = cross_sep

    def printTable(self, f, headers, table):
        # Compute length of the columns
        lengths = None
        for row in table:
            if lengths is None:
                lengths = {}
                for h in headers:
                    lengths[h] = len(str(h))
            for key, column in row.items():
                length = len(str(column))
                lengths[key] = max(lengths[key], length)

        # Header and separator
        header = self.vsep
        sep = self.cross_sep
        for key in headers:
            h = str(key).ljust(lengths[key], " ")
            header += " %s %s" % (h, self.vsep)
            sep += "%s%s" % (self.hsep * (len(h) + 2), self.cross_sep)
        f.write("%s\n" % sep)
        f.write("%s\n" % header)
        f.write("%s\n" % sep)

        for row in table:
            f.write(self.vsep)
            for key in headers:
                f.write(" %s %s" % (str(row[key]).ljust(lengths[key], " "), self.vsep))
            f.write("\n")
        f.write("%s\n" % sep)

class CQLConsole(object):
    def __init__(self, url, username, password, printer=RawPrinter()):
        self.url = url
        self.username = username
        self.password = password
        self.printer = printer

        self.connection = pywbem.WBEMConnection(url, (username, password))

    def runQuery(self, query):
        try:
            result = self.execQuery(query)
        except QueryException, e:
            print >>sys.stderr, "Unable to execute query:", str(e)
            return 1

        table, headers = self.processResult(result)
        if len(table) > 0:
            self.printTable(sys.stdout, headers, table)
        else:
            print >>sys.stderr, "No rows returned"
            return 1

        return 0

    def execQuery(self, query):
        try:
            return self.connection.ExecQuery('CQL', query)
        except Exception, e:
            raise QueryException(e[1])

    def processResult(self, result):
        table = []
        headers = []
        for item in result:
            row = {}
            for k in item.keys():
                if k not in headers:
                    headers.append(k)
                row[k] = item[k]
            table.append(row)
        return table, headers

    def printTable(self, f, headers, table):
        self.printer.printTable(f, headers, table)

if __name__ == '__main__':
    if len(sys.argv) < 2:
        print """This is interactive console for running CQL (CIM Query Language) queries on CIMOM server.

Usage: %s <address> [username] [password] [query]

If query is specified interactive console won't start and this program will exit immediatelly.

You can use this to pretty show wide results:
%s <address> username password query | less -S""" % (sys.argv[0], sys.argv[0])
        sys.exit(1)

    url = sys.argv[1]
    username = None
    password = None
    sql = None
    if len(sys.argv) > 2:
        username = sys.argv[2]
    if len(sys.argv) > 3:
        password = sys.argv[3]
    if len(sys.argv) > 4:
        sql = sys.argv[4]

    console = CQLConsole(url, username, password)
    if sql is not None:
        sys.exit(console.runQuery(sql))

    use_readline = False
    try:
        import readline
        use_readline = True
    except Exception:
        pass

    if use_readline:
        if os.access(HISTORY, os.R_OK):
            readline.read_history_file(HISTORY)
        else:
            f = open(HISTORY, "w")
            f.close()

    while True:
        try:
            sql = raw_input("> ")
        except EOFError:
            break

        if len(sql) == 0:
            continue

        console.runQuery(sql)

    if use_readline:
        readline.write_history_file(HISTORY)



#!/usr/bin/env python
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Grec <jgrec@redhat.com>
# Authors: Michal Minar <miminar@redhat.com>
#
"""
Unit tests for ``LMI_InstalledSoftwareIdentity`` provider.
"""

import pywbem
import unittest

from lmi.shell import LMIDeletedObjectError
from lmi.test.lmibase import enable_lmi_exceptions
from lmi.test.util import mark_tedious
import package
import swbase
import util

class TestInstalledSoftwareIdentity(swbase.SwTestCase):
    """
    Basic cim operations test on `LMI_InstalledSoftwareIdentity``.
    """

    CLASS_NAME = "LMI_InstalledSoftwareIdentity"
    KEYS = ("InstalledSoftware", "System")

    def make_op(self, pkg):
        """
        :returns Object path of ``LMI_InstalledSoftwareIdentity``.
        :rtype: :py:class:`lmi.shell.LMIInstanceName`
        """
        return self.cim_class.new_instance_name({
            "System" : self.system_iname,
            "InstalledSoftware" : self.ns.LMI_SoftwareIdentity. \
                new_instance_name({
                    "InstanceID" : 'LMI:LMI_SoftwareIdentity:' + pkg.nevra
                })
            })

    @swbase.test_with_packages('stable#pkg1')
    def test_get_instance(self):
        """
        Test ``GetInstance()`` call on ``LMI_InstalledSoftwareIdentity``.
        """
        repo = self.get_repo('stable')
        pkg = repo['pkg1']
        objpath = self.make_op(pkg)
        inst = objpath.to_instance()
        self.assertEqual(set(inst.path.key_properties()), set(self.KEYS))
        for key in self.KEYS:
            self.assertCIMNameEqual(
                    getattr(inst, key), getattr(inst.path, key))
        self.assertCIMNameEqual(inst.path, objpath,
                "Object paths should match for package %s" % pkg)

        # try it now with CIM_ComputerSystem
        system = objpath.System.copy()
        objpath.System.wrapped_object.classname = "CIM_ComputerSystem"
        objpath.System.wrapped_object.CreationClassName = \
                "CIM_ComputerSystem"

        inst = objpath.to_instance()
        self.assertCIMNameEqual(inst.path, objpath,
                "Object paths should match for package %s" % pkg)
        for key in self.KEYS:
            self.assertIn(key, inst.properties(),
                    "OP is missing \"%s\" key for package %s" % (key, pkg))
        self.assertCIMNameEqual(inst.System, system)
        self.assertCIMNameEqual(inst.path, objpath)

    @enable_lmi_exceptions
    @swbase.test_with_packages(**{'stable#pkg2' : False })
    def test_get_instance_not_installed(self):
        """
        Test ``GetInstance()`` call on ``LMI_InstalledSoftwareIdentity``.
        """
        repo = self.get_repo('stable')
        pkg = repo['pkg2']
        objpath = self.make_op(pkg)
        self.assertRaisesCIM(pywbem.CIM_ERR_NOT_FOUND, objpath.to_instance)

    @mark_tedious
    @swbase.test_with_packages(**{
        'pkg3' : False,
        'pkg4' : False,
        'stable#pkg1' : True,
        'stable#pkg2' : False,
        'updates#*' : False,
        'updates-testing#pkg1' : False,
        'updates-testing#pkg2' : True,
        'misc#conflict1' : False,
        'misc#conflict2' : False,
        'misc#depend1' : False,
        'misc#depend2' : False,
        'misc#pkg2' : False,
        'misc#funny-version' : True,
        'misc#funny-release' : True,
    })
    def test_enum_instance_names(self):
        """
        Test ``EnumInstanceNames()`` call on ``LMI_InstalledSoftwareIdentity``.
        """
        inames = self.cim_class.instance_names()
        self.assertGreater(len(inames), 0)
        objpath = self.make_op(self.get_repo('stable')['pkg1'])
        for iname in inames:
            self.assertEqual(iname.namespace, 'root/cimv2')
            self.assertEqual(iname.classname, self.CLASS_NAME)
            self.assertEqual(sorted(iname.key_properties()), sorted(self.KEYS))
            self.assertCIMNameEqual(objpath.System, iname.System)
        nevra_set = set(i.InstalledSoftware.InstanceID[
                len('LMI:LMI_SoftwareIdentity:'):] for i in inames)
        db_installed = 0
        for repo in self.repodb.values():
            for pkg in repo.packages:
                if package.is_pkg_installed(pkg):
                    self.assertTrue(pkg.nevra in nevra_set)
                    db_installed += 1
                else:
                    self.assertFalse(pkg.nevra in nevra_set)
        # test that number of tested packages installed really match number
        # of requested packages
        self.assertEqual(db_installed, 4)

    @mark_tedious
    @swbase.test_with_packages(**{
        'pkg3' : False,
        'pkg4' : False,
        'stable#pkg1' : True,
        'stable#pkg2' : False,
        'updates#*' : False,
        'updates-testing#pkg1' : False,
        'updates-testing#pkg2' : True,
        'misc#conflict1' : False,
        'misc#conflict2' : False,
        'misc#depend1' : False,
        'misc#depend2' : False,
        'misc#pkg2' : False,
        'misc#funny-version' : True,
        'misc#funny-release' : True,
    })
    def test_enum_instances(self):
        """
        Test ``EnumInstances() call on ``LMI_InstalledSoftwareIdentity``.
        """
        insts = self.cim_class.instances()
        self.assertGreater(len(insts), 0)
        for inst in insts:
            self.assertEqual(set(inst.path.key_properties()), set(self.KEYS))
            for key in self.KEYS:
                self.assertCIMNameEqual(
                        getattr(inst, key), getattr(inst.path, key))
        nevra_set = set(i.InstalledSoftware.InstanceID[
                len('LMI:LMI_SoftwareIdentity:'):] for i in insts)
        db_installed = 0
        for repo in self.repodb.values():
            for pkg in repo.packages:
                if package.is_pkg_installed(pkg):
                    self.assertTrue(pkg.nevra in nevra_set)
                    db_installed += 1
                else:
                    self.assertFalse(pkg.nevra in nevra_set)
        # test that number of tested packages installed really match number
        # of requested packages
        self.assertEqual(db_installed, 4)

    @mark_tedious
    def test_list_installed_packages(self):
        """
        Test list of all installed packages.

        Use LMI_InstalledSoftwareIdentity instances to get installed packages.
        Compare all package names with:
            rpm -qa
        Cases:
            OpenLMI package list matches yum package list.
        """
        package_list = []
        for iname in self.cim_class.instance_names():
            path = iname.InstalledSoftware.InstanceID
            name = path[len("LMI:LMI_SoftwareIdentity:"):]
            package_list.append(name)

        self.assertNotEqual(len(package_list), 0,
                            "OpenLMI returned empty list of installed packages")

        rpm_packages = util.get_installed_packages()
        self.assertEqual(sorted(package_list), sorted(rpm_packages))

    @swbase.test_with_repos('stable',
            **{'updates' : False, 'updates-testing' : False, 'misc' : False })
    @swbase.test_with_packages(**{'stable#pkg1' : False})
    def test_install_package(self):
        """
        Test package installation  with ``LMI_InstalledSoftwareIdentity``.
        """
        repo = self.get_repo('stable')
        pkg = repo['pkg1']
        self.assertFalse(package.is_pkg_installed(pkg))
        objpath = self.make_op(pkg)

        properties = {
                "InstalledSoftware" : objpath.InstalledSoftware,
                "System"            : objpath.System
            }

        inst = self.cim_class.create_instance(properties)
        self.assertTrue(package.is_pkg_installed(pkg))
        self.assertCIMNameEqual(inst.path, objpath,
                'Object path does not match for %s.' % pkg)

    @swbase.test_with_repos('stable', 'misc',
            **{ 'updates' : False, 'updates-testing' : False })
    @swbase.test_with_packages(**{'pkg2' : False})
    def test_install_package_same_arch(self):
        """
        Test package installation  with ``LMI_InstalledSoftwareIdentity`` of
        package with specific architecture.
        """
        pkg_arch = self.get_repo('misc')['pkg2']
        op_arch = self.make_op(pkg_arch)
        properties = {
                "InstalledSoftware" : op_arch.InstalledSoftware,
                "System"            : op_arch.System
            }

        inst = self.cim_class.create_instance(properties)
        self.assertTrue(package.is_pkg_installed(pkg_arch))
        self.assertCIMNameEqual(inst.path, op_arch,
                'Object path does not match for %s.' % pkg_arch)

    @enable_lmi_exceptions
    @swbase.test_with_repos('stable',
            **{'updates' : False, 'updates-testing' : False, 'misc' : False })
    @swbase.test_with_packages('stable#pkg1')
    def test_install_already_installed(self):
        """
        Try to install package that is already installed with
        ``LMI_InstalledSoftwareIdentity``.
        """
        pkg = self.get_repo('stable')['pkg1']
        self.assertTrue(package.is_pkg_installed(pkg))
        objpath = self.make_op(pkg)

        properties = {
                "InstalledSoftware" : objpath.InstalledSoftware,
                "System"            : objpath.System
            }
        # try to install second time
        self.assertRaisesCIM(pywbem.CIM_ERR_ALREADY_EXISTS,
            self.cim_class.create_instance, properties)

    @swbase.test_with_repos(**{'stable' : False})
    @swbase.test_with_packages(**{'stable#pkg1' : False})
    def test_install_from_disabled_repo(self):
        pkg = self.get_repo('stable')['pkg1']
        self.assertFalse(package.is_pkg_installed(pkg))
        objpath = self.make_op(pkg)

        properties = {
                "InstalledSoftware" : objpath.InstalledSoftware,
                "System"            : objpath.System
            }
        # try to install it
        inst = self.cim_class.create_instance(properties)
        self.assertEqual(inst, None)

    @enable_lmi_exceptions
    @swbase.test_with_packages('stable#pkg2')
    def test_remove_installed(self):
        """
        Test package removal with ``LMI_InstalledSoftwareIdentity``.
        """
        pkg = self.get_repo('stable')['pkg2']
        self.assertTrue(package.is_pkg_installed(pkg))
        objpath = self.make_op(pkg)
        inst = objpath.to_instance()
        inst.delete()
        self.assertFalse(package.is_pkg_installed(pkg))

        # try to delete second time
        self.assertRaises(LMIDeletedObjectError, inst.delete)
        self.assertRaisesCIM(pywbem.CIM_ERR_NOT_FOUND, objpath.to_instance)

    @mark_tedious
    @swbase.test_with_packages(**{
        'updates#pkg3' : True,
        'stable#pkg1' : True,
        'pkg2' : False,
        'pkg4' : False,
        'updates-testing#*' : False,
        'misc#*' : False
    })
    def test_get_installed_package_names(self):
        """
        Try to get installed packages associated with computer system
        via ``LMI_InstalledSoftwareIdentity``.
        """
        pkg = self.get_repo('stable')['pkg1']
        objpath = self.make_op(pkg)
        refs = objpath.System.to_instance().associator_names(
                AssocClass=self.CLASS_NAME,
                Role="System",
                ResultRole="InstalledSoftware",
                ResultClass="LMI_SoftwareIdentity")
        self.assertGreater(len(refs), 0)
        for ref in refs:
            self.assertEqual(ref.namespace, 'root/cimv2')
            self.assertEqual(ref.classname, "LMI_SoftwareIdentity")
            self.assertEqual(ref.key_properties(), ["InstanceID"])
            self.assertTrue(ref.InstanceID.startswith(
                "LMI:LMI_SoftwareIdentity:"))
        nevra_set = set(   i.InstanceID[len('LMI:LMI_SoftwareIdentity:'):]
                       for i in refs)

        db_installed = 0
        for repo in self.repodb.values():
            for pkg in repo.packages:
                if package.is_pkg_installed(pkg):
                    self.assertTrue(pkg.nevra in nevra_set)
                    db_installed += 1
                else:
                    self.assertFalse(pkg.nevra in nevra_set)
        # test that number of tested packages installed really match number
        # of requested packages
        self.assertEqual(db_installed, 2)

    @swbase.test_with_packages('stable#pkg1')
    def test_get_installed_software_referents(self):
        """
        Try to get computer system associated with installed package via
        ``LMI_InstalledSoftwareIdentity``.
        """
        pkg = self.get_repo('stable')['pkg1']
        self.assertTrue(package.is_pkg_installed(pkg))
        objpath = self.make_op(pkg)
        refs = objpath.InstalledSoftware.to_instance().associator_names(
                AssocClass='LMI_InstalledSoftwareIdentity',
                Role="InstalledSoftware",
                ResultRole="System",
                ResultClass=self.system_cs_name)
        self.assertEqual(len(refs), 1)
        ref = refs[0]
        self.assertCIMNameEqual(objpath.System, ref)

def suite():
    """For unittest loaders."""
    return unittest.TestLoader().loadTestsFromTestCase(
            TestInstalledSoftwareIdentity)

if __name__ == '__main__':
    unittest.main()
